"""
Unit tests for reference objects. This is used for all tests
that can be performed in isolation from input data.
"""
from __future__ import division
from __future__ import print_function
from __future__ import unicode_literals

import unittest

import ga4gh.backend as backend
import ga4gh.datamodel.references as references
import ga4gh.exceptions as exceptions
import ga4gh.datarepo as datarepo


class TestAbstractReferenceSet(unittest.TestCase):
    """
    Unit tests for the abstract reference set.
    """
    def setUp(self):
        self._backend = backend.Backend(datarepo.AbstractDataRepository())
        self._referenceSet = references.AbstractReferenceSet(
            'refSetId')

    def testAddOneReference(self):
        self.assertEqual(self._referenceSet.getNumReferences(), 0)
        referenceName = "ref"
        reference = references.AbstractReference(
            self._referenceSet, referenceName)
        self._referenceSet.addReference(reference)
        self.assertEqual(self._referenceSet.getNumReferences(), 1)
        self.assertEqual(
            self._referenceSet.getReferenceByIndex(0), reference)
        self.assertEqual(
            self._referenceSet.getReferenceByName(referenceName), reference)
        self.assertEqual(
            self._referenceSet.getReference(reference.getId()), reference)
        self.assertEqual(self._referenceSet.getReferences(), [reference])

    def testAddMultipleReference(self):
        referenceList = []
        referenceBaseName = "ref"
        referenceCount = 10

        self.assertEqual(self._referenceSet.getNumReferences(), 0)

        for i in range(referenceCount):
            referenceName = referenceBaseName + str(i)
            reference = references.AbstractReference(
                self._referenceSet, referenceName)
            referenceList.append(reference)
            self._referenceSet.addReference(reference)
            self.assertEqual(self._referenceSet.getNumReferences(), i + 1)
            self.assertEqual(
                self._referenceSet.getReferenceByIndex(i), reference)
            self.assertEqual(
                self._referenceSet.getReferenceByName(referenceName),
                reference)
            self.assertEqual(
                self._referenceSet.getReference(reference.getId()), reference)
            self.assertEqual(self._referenceSet.getReferences(), referenceList)

    def testReferenceNameNotFound(self):
        for badName in ["", None, "NO SUCH NAME"]:
            self.assertRaises(
                exceptions.ReferenceNameNotFoundException,
                self._referenceSet.getReferenceByName, badName)

    def testReferenceNotFound(self):
        for badId in ["", None, "NO SUCH ID"]:
            self.assertRaises(
                exceptions.ReferenceNotFoundException,
                self._referenceSet.getReference, badId)


class TestAbstractReference(unittest.TestCase):
    """
    Unit tests for the abstract reference object.
    """
    def setUp(self):
        self._backend = backend.Backend(
            datarepo.AbstractDataRepository())
        self._referenceSet = references.AbstractReferenceSet(
            'refSetId')
        self._reference = references.AbstractReference(
            self._referenceSet, "ref")

    def testReferenceRangeError(self):
        for badRange in [(-1, 3), (100, 50), (100, 1000)]:
            self.assertRaises(
                exceptions.ReferenceRangeErrorException,
                self._reference.checkQueryRange, badRange[0], badRange[1])



# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: ga4gh/allele_annotation_service.proto

import sys
_b=sys.version_info[0]<3 and (lambda x:x) or (lambda x:x.encode('latin1'))
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
from google.protobuf import descriptor_pb2
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from ga4gh import allele_annotations_pb2 as ga4gh_dot_allele__annotations__pb2
from ga4gh import metadata_pb2 as ga4gh_dot_metadata__pb2


DESCRIPTOR = _descriptor.FileDescriptor(
  name='ga4gh/allele_annotation_service.proto',
  package='ga4gh',
  syntax='proto3',
  serialized_pb=_b('\n%ga4gh/allele_annotation_service.proto\x12\x05ga4gh\x1a\x1ega4gh/allele_annotations.proto\x1a\x14ga4gh/metadata.proto\"\xdb\x01\n\x1fSearchVariantAnnotationsRequest\x12!\n\x19variant_annotation_set_id\x18\x01 \x01(\t\x12\x16\n\x0ereference_name\x18\x02 \x01(\t\x12\x14\n\x0creference_id\x18\x03 \x01(\t\x12\r\n\x05start\x18\x04 \x01(\x03\x12\x0b\n\x03\x65nd\x18\x05 \x01(\x03\x12$\n\x07\x65\x66\x66\x65\x63ts\x18\x06 \x03(\x0b\x32\x13.ga4gh.OntologyTerm\x12\x11\n\tpage_size\x18\x07 \x01(\x05\x12\x12\n\npage_token\x18\x08 \x01(\t\"r\n SearchVariantAnnotationsResponse\x12\x35\n\x13variant_annotations\x18\x01 \x03(\x0b\x32\x18.ga4gh.VariantAnnotation\x12\x17\n\x0fnext_page_token\x18\x02 \x01(\t\"c\n\"SearchVariantAnnotationSetsRequest\x12\x16\n\x0evariant_set_id\x18\x01 \x01(\t\x12\x11\n\tpage_size\x18\x02 \x01(\x05\x12\x12\n\npage_token\x18\x03 \x01(\t\"|\n#SearchVariantAnnotationSetsResponse\x12<\n\x17variant_annotation_sets\x18\x01 \x03(\x0b\x32\x1b.ga4gh.VariantAnnotationSet\x12\x17\n\x0fnext_page_token\x18\x02 \x01(\t\"C\n\x1eGetVariantAnnotationSetRequest\x12!\n\x19variant_annotation_set_id\x18\x01 \x01(\t2\xdb\x02\n\x17\x41lleleAnnotationService\x12k\n\x18SearchVariantAnnotations\x12&.ga4gh.SearchVariantAnnotationsRequest\x1a\'.ga4gh.SearchVariantAnnotationsResponse\x12t\n\x1bSearchVariantAnnotationSets\x12).ga4gh.SearchVariantAnnotationSetsRequest\x1a*.ga4gh.SearchVariantAnnotationSetsResponse\x12]\n\x17GetVariantAnnotationSet\x12%.ga4gh.GetVariantAnnotationSetRequest\x1a\x1b.ga4gh.VariantAnnotationSetb\x06proto3')
  ,
  dependencies=[ga4gh_dot_allele__annotations__pb2.DESCRIPTOR,ga4gh_dot_metadata__pb2.DESCRIPTOR,])
_sym_db.RegisterFileDescriptor(DESCRIPTOR)




_SEARCHVARIANTANNOTATIONSREQUEST = _descriptor.Descriptor(
  name='SearchVariantAnnotationsRequest',
  full_name='ga4gh.SearchVariantAnnotationsRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='variant_annotation_set_id', full_name='ga4gh.SearchVariantAnnotationsRequest.variant_annotation_set_id', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='reference_name', full_name='ga4gh.SearchVariantAnnotationsRequest.reference_name', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='reference_id', full_name='ga4gh.SearchVariantAnnotationsRequest.reference_id', index=2,
      number=3, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='start', full_name='ga4gh.SearchVariantAnnotationsRequest.start', index=3,
      number=4, type=3, cpp_type=2, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='end', full_name='ga4gh.SearchVariantAnnotationsRequest.end', index=4,
      number=5, type=3, cpp_type=2, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='effects', full_name='ga4gh.SearchVariantAnnotationsRequest.effects', index=5,
      number=6, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='page_size', full_name='ga4gh.SearchVariantAnnotationsRequest.page_size', index=6,
      number=7, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='page_token', full_name='ga4gh.SearchVariantAnnotationsRequest.page_token', index=7,
      number=8, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=103,
  serialized_end=322,
)


_SEARCHVARIANTANNOTATIONSRESPONSE = _descriptor.Descriptor(
  name='SearchVariantAnnotationsResponse',
  full_name='ga4gh.SearchVariantAnnotationsResponse',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='variant_annotations', full_name='ga4gh.SearchVariantAnnotationsResponse.variant_annotations', index=0,
      number=1, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='next_page_token', full_name='ga4gh.SearchVariantAnnotationsResponse.next_page_token', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=324,
  serialized_end=438,
)


_SEARCHVARIANTANNOTATIONSETSREQUEST = _descriptor.Descriptor(
  name='SearchVariantAnnotationSetsRequest',
  full_name='ga4gh.SearchVariantAnnotationSetsRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='variant_set_id', full_name='ga4gh.SearchVariantAnnotationSetsRequest.variant_set_id', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='page_size', full_name='ga4gh.SearchVariantAnnotationSetsRequest.page_size', index=1,
      number=2, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='page_token', full_name='ga4gh.SearchVariantAnnotationSetsRequest.page_token', index=2,
      number=3, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=440,
  serialized_end=539,
)


_SEARCHVARIANTANNOTATIONSETSRESPONSE = _descriptor.Descriptor(
  name='SearchVariantAnnotationSetsResponse',
  full_name='ga4gh.SearchVariantAnnotationSetsResponse',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='variant_annotation_sets', full_name='ga4gh.SearchVariantAnnotationSetsResponse.variant_annotation_sets', index=0,
      number=1, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='next_page_token', full_name='ga4gh.SearchVariantAnnotationSetsResponse.next_page_token', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=541,
  serialized_end=665,
)


_GETVARIANTANNOTATIONSETREQUEST = _descriptor.Descriptor(
  name='GetVariantAnnotationSetRequest',
  full_name='ga4gh.GetVariantAnnotationSetRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='variant_annotation_set_id', full_name='ga4gh.GetVariantAnnotationSetRequest.variant_annotation_set_id', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=667,
  serialized_end=734,
)

_SEARCHVARIANTANNOTATIONSREQUEST.fields_by_name['effects'].message_type = ga4gh_dot_metadata__pb2._ONTOLOGYTERM
_SEARCHVARIANTANNOTATIONSRESPONSE.fields_by_name['variant_annotations'].message_type = ga4gh_dot_allele__annotations__pb2._VARIANTANNOTATION
_SEARCHVARIANTANNOTATIONSETSRESPONSE.fields_by_name['variant_annotation_sets'].message_type = ga4gh_dot_allele__annotations__pb2._VARIANTANNOTATIONSET
DESCRIPTOR.message_types_by_name['SearchVariantAnnotationsRequest'] = _SEARCHVARIANTANNOTATIONSREQUEST
DESCRIPTOR.message_types_by_name['SearchVariantAnnotationsResponse'] = _SEARCHVARIANTANNOTATIONSRESPONSE
DESCRIPTOR.message_types_by_name['SearchVariantAnnotationSetsRequest'] = _SEARCHVARIANTANNOTATIONSETSREQUEST
DESCRIPTOR.message_types_by_name['SearchVariantAnnotationSetsResponse'] = _SEARCHVARIANTANNOTATIONSETSRESPONSE
DESCRIPTOR.message_types_by_name['GetVariantAnnotationSetRequest'] = _GETVARIANTANNOTATIONSETREQUEST

SearchVariantAnnotationsRequest = _reflection.GeneratedProtocolMessageType('SearchVariantAnnotationsRequest', (_message.Message,), dict(
  DESCRIPTOR = _SEARCHVARIANTANNOTATIONSREQUEST,
  __module__ = 'ga4gh.allele_annotation_service_pb2'
  # @@protoc_insertion_point(class_scope:ga4gh.SearchVariantAnnotationsRequest)
  ))
_sym_db.RegisterMessage(SearchVariantAnnotationsRequest)

SearchVariantAnnotationsResponse = _reflection.GeneratedProtocolMessageType('SearchVariantAnnotationsResponse', (_message.Message,), dict(
  DESCRIPTOR = _SEARCHVARIANTANNOTATIONSRESPONSE,
  __module__ = 'ga4gh.allele_annotation_service_pb2'
  # @@protoc_insertion_point(class_scope:ga4gh.SearchVariantAnnotationsResponse)
  ))
_sym_db.RegisterMessage(SearchVariantAnnotationsResponse)

SearchVariantAnnotationSetsRequest = _reflection.GeneratedProtocolMessageType('SearchVariantAnnotationSetsRequest', (_message.Message,), dict(
  DESCRIPTOR = _SEARCHVARIANTANNOTATIONSETSREQUEST,
  __module__ = 'ga4gh.allele_annotation_service_pb2'
  # @@protoc_insertion_point(class_scope:ga4gh.SearchVariantAnnotationSetsRequest)
  ))
_sym_db.RegisterMessage(SearchVariantAnnotationSetsRequest)

SearchVariantAnnotationSetsResponse = _reflection.GeneratedProtocolMessageType('SearchVariantAnnotationSetsResponse', (_message.Message,), dict(
  DESCRIPTOR = _SEARCHVARIANTANNOTATIONSETSRESPONSE,
  __module__ = 'ga4gh.allele_annotation_service_pb2'
  # @@protoc_insertion_point(class_scope:ga4gh.SearchVariantAnnotationSetsResponse)
  ))
_sym_db.RegisterMessage(SearchVariantAnnotationSetsResponse)

GetVariantAnnotationSetRequest = _reflection.GeneratedProtocolMessageType('GetVariantAnnotationSetRequest', (_message.Message,), dict(
  DESCRIPTOR = _GETVARIANTANNOTATIONSETREQUEST,
  __module__ = 'ga4gh.allele_annotation_service_pb2'
  # @@protoc_insertion_point(class_scope:ga4gh.GetVariantAnnotationSetRequest)
  ))
_sym_db.RegisterMessage(GetVariantAnnotationSetRequest)


# @@protoc_insertion_point(module_scope)



"""
ga2sam cli
"""
from __future__ import division
from __future__ import print_function
from __future__ import unicode_literals

import ga4gh.cli as cli
import ga4gh.cli.client as cli_client
import ga4gh.converters as converters


class Ga2SamRunner(cli_client.SearchReadsRunner):
    """
    Runner class for the ga2vcf
    """
    def __init__(self, args):
        args.readGroupIds = args.readGroupId
        super(Ga2SamRunner, self).__init__(args)
        self._outputFile = args.outputFile
        self._binaryOutput = False
        if args.outputFormat == "bam":
            self._binaryOutput = True

    def run(self):
        samConverter = converters.SamConverter(
            self._client, readGroupId=self._readGroupIds[0],
            referenceId=self._referenceId, start=self._start, end=self._end,
            outputFileName=self._outputFile, binaryOutput=self._binaryOutput)
        samConverter.convert()


def getGa2SamParser():
    parser = cli.createArgumentParser("GA4GH SAM conversion tool")
    cli_client.addClientGlobalOptions(parser)
    cli_client.addUrlArgument(parser)
    parser.add_argument(
        "readGroupId",
        help="The ReadGroup to convert to SAM/BAM format.")
    cli_client.addPageSizeArgument(parser)
    cli_client.addStartArgument(parser)
    cli_client.addEndArgument(parser)
    parser.add_argument(
        "--referenceId", default=None,
        help="The referenceId to search over")
    parser.add_argument(
        "--outputFormat", "-O", default="sam", choices=["sam", "bam"],
        help=(
            "The format for object output. Currently supported are "
            "'sam' (default), which is a text-based format and "
            "'bam', which is the binary equivalent"))
    cli.addOutputFileArgument(parser)
    return parser


def ga2sam_main():
    parser = getGa2SamParser()
    args = parser.parse_args()
    if "baseUrl" not in args:
        parser.print_help()
    else:
        runner = Ga2SamRunner(args)
        runner.run()



"""
Centralizes hardcoded paths, names, etc. used in tests
"""
from __future__ import division
from __future__ import print_function
from __future__ import unicode_literals

import os


testDir = 'tests'
testDataDir = os.path.join(testDir, 'data')
testDataRepo = os.path.join(testDataDir, 'registry.db')

# datasets
datasetName = "dataset1"
datasetsDir = os.path.join(testDataDir, "datasets")
datasetDir = os.path.join(datasetsDir, "dataset1")

# references
referenceSetName = 'chr17'
referenceSetsDir = os.path.join(testDataDir, 'referenceSets')
faPath = os.path.join(referenceSetsDir, 'Default.fa.gz')
faPath2 = os.path.join(referenceSetsDir, 'example_1.fa.gz')
faPath3 = os.path.join(referenceSetsDir, 'example_2.fa.gz')
ncbi37FaPath = os.path.join(referenceSetsDir, 'NCBI37.fa.gz')

# variants
variantSetName = '1kgPhase1'
variantsDir = os.path.join(datasetDir, 'variants')
vcfDirPath = os.path.join(variantsDir, '1kgPhase1')
vcfDirPath2 = os.path.join(variantsDir, '1kgPhase3')
vcfPath1 = os.path.join(vcfDirPath, 'chr1.vcf.gz')
vcfPath2 = os.path.join(vcfDirPath, 'chr2.vcf.gz')
vcfIndexPath1 = os.path.join(vcfDirPath, 'chr1.vcf.gz.tbi')
vcfIndexPath2 = os.path.join(vcfDirPath, 'chr2.vcf.gz.tbi')
annotatedVcfPath = os.path.join(variantsDir, '1kg.3.annotations')

# Ontologies
ontologyName = "so-xp-simple"
ontologiesDir = os.path.join(testDataDir, 'ontologies')
ontologyPath = os.path.join(ontologiesDir, 'so-xp-simple.obo')

# reads
readGroupSetName = 'chr17.1-250'
bamDir = os.path.join(datasetDir, 'reads')
bamPath = os.path.join(bamDir, 'chr17.1-250.bam')
bamPath2 = os.path.join(
    bamDir, 'wgEncodeUwRepliSeqBg02esG1bAlnRep1_sample.bam')
bamIndexPath = os.path.join(bamDir, 'chr17.1-250.bam.bai')
bamIndexPath2 = os.path.join(
    bamDir, 'wgEncodeUwRepliSeqBg02esG1bAlnRep1_sample.bam.bai')

# sequence annotations
featureSetName = 'gencodeV21Set1'
featuresDir = os.path.join(datasetDir, 'sequenceAnnotations')
featuresPath = os.path.join(featuresDir, 'gencodeV21Set1.db')
featuresPath2 = os.path.join(featuresDir, 'specialCasesTest.db')

# g2p
phenotypesDir = os.path.join(datasetDir, 'phenotypes')
phenotypeAssociationSetPath = os.path.join(phenotypesDir, 'cgd')

# rna
rnaQuantDir = os.path.join(datasetDir, "rnaQuant")
rnaQuantificationSetName = "ENCFF305LZB"
rnaQuantificationSetDbPath = os.path.join(rnaQuantDir, "ENCFF305LZB.db")

# misc.
landingMessageHtml = os.path.join(testDataDir, "test.html")



"""
Unit tests for test utility functions.
"""
from __future__ import division
from __future__ import print_function
from __future__ import unicode_literals

import unittest
import sys
import time

import tests.utils as utils


class TestRepeat(unittest.TestCase):
    """
    Test the Repeat decorator
    """
    # lowest possible positive number
    sleepSeconds = sys.float_info.min

    @utils.Repeat(sleepSeconds)
    def repeating(self):
        self.i += 1
        return self.i < self.endIterationAt

    def testRepeat(self):
        self.i = 0
        self.endIterationAt = 3
        self.repeating()
        self.assertEqual(self.i, self.endIterationAt)


class TestTimeout(unittest.TestCase):
    """
    Test the Timeout decorator
    """
    # lowest positive value signal.alarm allows
    timeoutSecondsLow = 1

    @utils.Timeout(timeoutSecondsLow)
    def timingOut(self):
        time.sleep(60)  # much longer than the timeout

    @utils.Timeout()
    def notTimingOut(self):
        return 0

    def testTimeoutException(self):
        with self.assertRaises(utils.TimeoutException):
            self.timingOut()

    def testTimeoutNoException(self):
        self.assertEquals(self.notTimingOut(), 0)
        # no exception thrown


class TestCaptureOutput(unittest.TestCase):
    """
    Test that the captureOutput correctly returns the value of stdout
    and stderr for a function.
    """

    def testCapture(self):
        stdoutValue = "stdout"
        stderrValue = "stderr"

        def func():
            print(stdoutValue, file=sys.stdout, end="")
            print(stderrValue, file=sys.stderr, end="")
        stdout, stderr = utils.captureOutput(func)
        self.assertEqual(stdout, stdoutValue)
        self.assertEqual(stderr, stderrValue)

        # Empty stdout
        def func():
            print(stderrValue, file=sys.stderr, end="")
        stdout, stderr = utils.captureOutput(func)
        self.assertEqual(stdout, "")
        self.assertEqual(stderr, stderrValue)

        # Empty stderr
        def func():
            print(stdoutValue, file=sys.stdout, end="")
        stdout, stderr = utils.captureOutput(func)
        self.assertEqual(stdout, stdoutValue)
        self.assertEqual(stderr, "")

    def testArgs(self):
        def func(one, two, three, keywordOne=None, keywordTwo=None):
            print(one, two, three, keywordOne, keywordTwo, file=sys.stdout)
        stdout, stderr = utils.captureOutput(
            func, "1", "2", "3", keywordTwo="5", keywordOne="4")
        self.assertEqual(stdout, "1 2 3 4 5\n")
        self.assertEqual(stderr, "")



"""
An end to end test which tests:
- client cmd line parsing
- client operation
- client logging
- server cmd line parsing
- server operation
- simulated variantSet backend
- server logging
"""
from __future__ import division
from __future__ import print_function
from __future__ import unicode_literals

import server_test
import client

import ga4gh.datarepo as datarepo


class TestGestalt(server_test.ServerTest):
    """
    An end-to-end test of the client and server
    """
    def testEndToEnd(self):
        # extract ids from a simulated data repo with the same config
        repo = datarepo.SimulatedDataRepository()
        dataset = repo.getDatasets()[0]
        datasetId = dataset.getId()
        variantSet = dataset.getVariantSets()[0]
        variantSetId = variantSet.getId()
        readGroupSet = dataset.getReadGroupSets()[0]
        readGroupId = readGroupSet.getReadGroups()[0].getId()
        referenceSet = repo.getReferenceSets()[0]
        referenceSetId = referenceSet.getId()
        referenceId = referenceSet.getReferences()[0].getId()
        variantAnnotationSetId = \
            variantSet.getVariantAnnotationSets()[0].getId()

        self.simulatedDatasetId = datasetId
        self.simulatedVariantSetId = variantSetId
        self.simulatedReadGroupId = readGroupId
        self.simulatedReferenceSetId = referenceSetId
        self.simulatedReferenceId = referenceId
        self.simulatedVariantAnnotationSetId = variantAnnotationSetId
        self.client = client.ClientForTesting(self.server.getUrl())
        self.runVariantsRequest()
        self.assertLogsWritten()
        self.runReadsRequest()
        self.runReferencesRequest()
        self.runVariantSetsRequestDatasetTwo()
        self.runVariantAnnotationsRequest()
        self.runGetVariantAnnotationSetsRequest()
        self.client.cleanup()

    def assertLogsWritten(self):
        serverOutLines = self.server.getOutLines()
        serverErrLines = self.server.getErrLines()
        clientOutLines = self.client.getOutLines()
        clientErrLines = self.client.getErrLines()

        # nothing should be written to server stdout
        self.assertEqual(
            [], serverOutLines,
            "Server stdout log not empty")

        # server stderr should log at least one response success
        responseFound = False
        for line in serverErrLines:
            if ' 200 ' in line:
                responseFound = True
                break
        self.assertTrue(
            responseFound,
            "No successful server response logged to stderr")

        # client stdout should not be empty
        self.assertNotEqual(
            [], clientOutLines,
            "Client stdout log is empty")

        # number of variants to expect
        expectedNumClientOutLines = 2
        self.assertEqual(len(clientOutLines), expectedNumClientOutLines)

        # client stderr should log at least one post
        requestFound = False
        for line in clientErrLines:
            if 'POST' in line:
                requestFound = True
                break
        self.assertTrue(
            requestFound,
            "No request logged from the client to stderr")

    def runVariantsRequest(self):
        self.runClientCmd(
            self.client,
            "variants-search",
            "-s 0 -e 2 -V {}".format(self.simulatedVariantSetId))

    def runVariantAnnotationsRequest(self):
        self.runClientCmd(
            self.client,
            "variantannotations-search",
            "--variantAnnotationSetId {} -s 0 -e 2".format(
                self.simulatedVariantAnnotationSetId))

    def runGetVariantAnnotationSetsRequest(self):
        self.runClientCmd(
            self.client,
            "variantannotationsets-get",
            "{}".format(self.simulatedVariantAnnotationSetId))

    def runReadsRequest(self):
        args = "--readGroupIds {} --referenceId {}".format(
            self.simulatedReadGroupId, self.simulatedReferenceId)
        self.runClientCmd(self.client, "reads-search", args)

    def runReferencesRequest(self):
        referenceSetId = self.simulatedReferenceSetId
        referenceId = self.simulatedReferenceId
        cmd = "referencesets-search"
        self.runClientCmd(self.client, cmd)
        cmd = "references-search"
        args = "--referenceSetId={}".format(referenceSetId)
        self.runClientCmd(self.client, cmd, args)
        cmd = "referencesets-get"
        args = "{}".format(referenceSetId)
        self.runClientCmd(self.client, cmd, args)
        cmd = "references-get"
        args = "{}".format(referenceId)
        self.runClientCmd(self.client, cmd, args)
        cmd = "references-list-bases"
        args = "{}".format(referenceId)
        self.runClientCmd(self.client, cmd, args)

    def runVariantSetsRequestDatasetTwo(self):
        cmd = "variantsets-search"
        args = "--datasetId {}".format(self.simulatedDatasetId)
        self.runClientCmd(self.client, cmd, args)



"""
Unit tests for reads objects. This is used for all tests
that can be performed in isolation from input data.
"""
from __future__ import division
from __future__ import print_function
from __future__ import unicode_literals

import unittest

import ga4gh.datamodel.reads as reads
import ga4gh.protocol as protocol


class TestParseMalformedBamHeader(unittest.TestCase):
    """
    Tests for parsing of malformed bam headers.

    reads.parseMalformedBamHeader should not modify correct parsed
    headers and should parse out additional fields separated by spaces
    (instead of tabs as defined in the SAM spec).
    """

    def testGoodHeaderUnmodified(self):
        header = {'SO': 'coordinate', 'VN': '1.0'}
        self.assertEqual(header, reads.parseMalformedBamHeader(header))

    def testOriginalTypesUnmodified(self):
        # note real field tags, just checking that types are preserved
        header = {'int': 2845856850,
                  'float': 206.6,
                  'bool': True,
                  'string': '123'}
        self.assertEqual(header, reads.parseMalformedBamHeader(header))

    def testCommandsWithSpacesNotParsed(self):
        header = {'CL': 'bwa aln -q 15 -f $sai_file ' +
                        '$reference_fasta $fastq_file\tPP:bwa_index',
                  'ID': 'bwa_aln_fastq',
                  'PN': 'bwa',
                  'VN': '0.5.9-r16'}
        self.assertEqual(header, reads.parseMalformedBamHeader(header))

    def testSpaceSeparatedUnparsedFieldsParsed(self):
        header = {'LN': 249250621,
                  'M5': '1b22b98cdeb4a9304cb5d48026a85128',
                  'SN': '1',
                  'UR': 'ftp://ftp.1000genomes.ebi.ac.uk/vol1/ftp/' +
                        'technical/reference/phase2_reference_assembly' +
                        '_sequence/hs37d5.fa.gz        AS:NCBI37' +
                        '       SP:Human'}
        expected = {'LN': 249250621,
                    'M5': '1b22b98cdeb4a9304cb5d48026a85128',
                    'SN': '1',
                    'UR': 'ftp://ftp.1000genomes.ebi.ac.uk/vol1/ftp/' +
                          'technical/reference/phase2_reference_assembly' +
                          '_sequence/hs37d5.fa.gz',
                    'AS': 'NCBI37',
                    'SP': 'Human'}
        self.assertEqual(expected, reads.parseMalformedBamHeader(header))


class TestSamCigar(unittest.TestCase):
    """
    Test Sam Cigar class handles Cigar mappings correctly

    The integer codes are defined in the SAM spec. Thus, the ordering of
    SamCigar.cigarStrings implicitly implements this spec.
    """

    def testAlignmentMatch(self):
        self.assertEqual(0, reads.SamCigar.ga2int(
            protocol.CigarUnit.ALIGNMENT_MATCH))

        self.assertEqual(protocol.CigarUnit.ALIGNMENT_MATCH,
                         reads.SamCigar.int2ga(0))

    def testInsertion(self):
        self.assertEqual(1, reads.SamCigar.ga2int(
            protocol.CigarUnit.INSERT))

        self.assertEqual(protocol.CigarUnit.INSERT,
                         reads.SamCigar.int2ga(1))

    def testDeletion(self):
        self.assertEqual(2, reads.SamCigar.ga2int(
            protocol.CigarUnit.DELETE))

        self.assertEqual(protocol.CigarUnit.DELETE,
                         reads.SamCigar.int2ga(2))

    def testSkipped(self):
        self.assertEqual(3, reads.SamCigar.ga2int(
            protocol.CigarUnit.SKIP))

        self.assertEqual(protocol.CigarUnit.SKIP,
                         reads.SamCigar.int2ga(3))

    def testSoftClipping(self):
        self.assertEqual(4, reads.SamCigar.ga2int(
            protocol.CigarUnit.CLIP_SOFT))

        self.assertEqual(protocol.CigarUnit.CLIP_SOFT,
                         reads.SamCigar.int2ga(4))

    def testHardClipping(self):
        self.assertEqual(5, reads.SamCigar.ga2int(
            protocol.CigarUnit.CLIP_HARD))

        self.assertEqual(protocol.CigarUnit.CLIP_HARD,
                         reads.SamCigar.int2ga(5))

    def testPadding(self):
        self.assertEqual(6, reads.SamCigar.ga2int(
            protocol.CigarUnit.PAD))

        self.assertEqual(protocol.CigarUnit.PAD,
                         reads.SamCigar.int2ga(6))

    def testSequenceMatch(self):
        self.assertEqual(7, reads.SamCigar.ga2int(
            protocol.CigarUnit.SEQUENCE_MATCH))

        self.assertEqual(protocol.CigarUnit.SEQUENCE_MATCH,
                         reads.SamCigar.int2ga(7))

    def testSequenceMismatch(self):
        self.assertEqual(8, reads.SamCigar.ga2int(
            protocol.CigarUnit.SEQUENCE_MISMATCH))

        self.assertEqual(protocol.CigarUnit.SEQUENCE_MISMATCH,
                         reads.SamCigar.int2ga(8))


class TestSamFlags(unittest.TestCase):
    """
    Tests SamFlags utilities for checking the status of and
    setting flags.

    Flags are defined by the SAM spec.
    """

    def setUp(self):
        self.flag = 0x0

    def testPairedReadFlag(self):
        self.flag = reads.SamFlags.setFlag(
            self.flag, reads.SamFlags.READ_PAIRED)
        self.assertEqual(0x1, self.flag)
        self.assertTrue(reads.SamFlags.isFlagSet(
            self.flag, reads.SamFlags.READ_PAIRED))

    def testProperPairReadFlag(self):
        self.flag = reads.SamFlags.setFlag(
            self.flag, reads.SamFlags.READ_PROPER_PAIR)
        self.assertEqual(0x2, self.flag)
        self.assertTrue(reads.SamFlags.isFlagSet(
            self.flag, reads.SamFlags.READ_PROPER_PAIR))

    def testUnmappedReadFlag(self):
        self.flag = reads.SamFlags.setFlag(
            self.flag, reads.SamFlags.READ_UNMAPPED)
        self.assertEqual(0x4, self.flag)
        self.assertTrue(reads.SamFlags.isFlagSet(
            self.flag, reads.SamFlags.READ_UNMAPPED))

    def testUnmappedMateFlag(self):
        self.flag = reads.SamFlags.setFlag(
            self.flag, reads.SamFlags.MATE_UNMAPPED)
        self.assertEqual(0x8, self.flag)
        self.assertTrue(reads.SamFlags.isFlagSet(
            self.flag, reads.SamFlags.MATE_UNMAPPED))

    def testReverseStrandReadFlag(self):
        self.flag = reads.SamFlags.setFlag(
            self.flag, reads.SamFlags.READ_REVERSE_STRAND)
        self.assertEqual(0x10, self.flag)
        self.assertTrue(reads.SamFlags.isFlagSet(
            self.flag, reads.SamFlags.READ_REVERSE_STRAND))

    def testReverseStrandMateFlag(self):
        self.flag = reads.SamFlags.setFlag(
            self.flag, reads.SamFlags.MATE_REVERSE_STRAND)
        self.assertEqual(0x20, self.flag)
        self.assertTrue(reads.SamFlags.isFlagSet(
            self.flag, reads.SamFlags.MATE_REVERSE_STRAND))

    def testFirstPairFlag(self):
        self.flag = reads.SamFlags.setFlag(
            self.flag, reads.SamFlags.FIRST_IN_PAIR)
        self.assertEqual(0x40, self.flag)
        self.assertTrue(reads.SamFlags.isFlagSet(
            self.flag, reads.SamFlags.FIRST_IN_PAIR))

    def testSecondPairFlag(self):
        self.flag = reads.SamFlags.setFlag(
            self.flag, reads.SamFlags.SECOND_IN_PAIR)
        self.assertEqual(0x80, self.flag)
        self.assertTrue(reads.SamFlags.isFlagSet(
            self.flag, reads.SamFlags.SECOND_IN_PAIR))

    def testSecondaryAlignmentFlag(self):
        self.flag = reads.SamFlags.setFlag(
            self.flag, reads.SamFlags.SECONDARY_ALIGNMENT)
        self.assertEqual(0x100, self.flag)
        self.assertTrue(reads.SamFlags.isFlagSet(
            self.flag, reads.SamFlags.SECONDARY_ALIGNMENT))

    def testFailedQualityCheckFlag(self):
        self.flag = reads.SamFlags.setFlag(
            self.flag, reads.SamFlags.FAILED_QUALITY_CHECK)
        self.assertEqual(0x200, self.flag)
        self.assertTrue(reads.SamFlags.isFlagSet(
            self.flag, reads.SamFlags.FAILED_QUALITY_CHECK))

    def testDuplicateReadFlag(self):
        self.flag = reads.SamFlags.setFlag(
            self.flag, reads.SamFlags.DUPLICATE_READ)
        self.assertEqual(0x400, self.flag)
        self.assertTrue(reads.SamFlags.isFlagSet(
            self.flag, reads.SamFlags.DUPLICATE_READ))

    def testSupplementaryAlignmentFlag(self):
        self.flag = reads.SamFlags.setFlag(
            self.flag, reads.SamFlags.SUPPLEMENTARY_ALIGNMENT)
        self.assertEqual(0x800, self.flag)
        self.assertTrue(reads.SamFlags.isFlagSet(
            self.flag, reads.SamFlags.SUPPLEMENTARY_ALIGNMENT))

    def testFlagNotSet(self):
        self.flag = reads.SamFlags.setFlag(
            self.flag, reads.SamFlags.READ_PAIRED)
        self.assertFalse(reads.SamFlags.isFlagSet(
            self.flag, reads.SamFlags.READ_REVERSE_STRAND))

    def testComboFlag(self):
        self.flag = reads.SamFlags.setFlag(
            self.flag, reads.SamFlags.READ_PAIRED)
        self.flag = reads.SamFlags.setFlag(
            self.flag, reads.SamFlags.FIRST_IN_PAIR)
        self.flag = reads.SamFlags.setFlag(
            self.flag, reads.SamFlags.FAILED_QUALITY_CHECK)
        self.assertEqual(0x241, self.flag)
        self.assertTrue(reads.SamFlags.isFlagSet(
            self.flag, reads.SamFlags.READ_PAIRED))
        self.assertTrue(reads.SamFlags.isFlagSet(
            self.flag, reads.SamFlags.FIRST_IN_PAIR))
        self.assertTrue(reads.SamFlags.isFlagSet(
            self.flag, reads.SamFlags.FAILED_QUALITY_CHECK))



"""
Unit tests for variant objects. This is used for all tests
that can be performed in isolation from input data.
"""
from __future__ import division
from __future__ import print_function
from __future__ import unicode_literals

import hashlib
import unittest

import ga4gh.protocol as protocol
import ga4gh.datarepo as datarepo
import ga4gh.datamodel.variants as variants
import ga4gh.datamodel.datasets as datasets

import tests.paths as paths


class TestHtslibVariantAnnotationSet(unittest.TestCase):
    """
    Unit tests for the abstract variant set.
    """
    def _createVariantAnnotationSet(self, vcfDir):
        """
        Creates a VariantAnnotationSet from the specified directory of
        VCF files.
        """
        self._variantSetName = "testVariantSet"
        self._repo = datarepo.SqlDataRepository(paths.testDataRepo)
        self._repo.open(datarepo.MODE_READ)
        self._dataset = datasets.Dataset("testDs")
        self._variantSet = variants.HtslibVariantSet(
            self._dataset, self._variantSetName)
        self._variantSet.populateFromDirectory(vcfDir)
        self._variantAnnotationSet = variants.HtslibVariantAnnotationSet(
            self._variantSet, "testVAs")
        self._variantAnnotationSet.setOntology(
            self._repo.getOntologyByName(paths.ontologyName))

    def setUp(self):
        vcfDir = "tests/data/datasets/dataset1/variants/WASH7P_annotation"
        self._createVariantAnnotationSet(vcfDir)

    def testConvertLocation(self):
        loc = protocol.AlleleLocation()
        loc.start = 150
        pos = "151/305"
        testLoc = self._variantAnnotationSet.convertLocation(pos)
        self.assertEqual(testLoc, loc)

    def testThousandGenomesAnnotation(self):
        vcfDir = "tests/data/datasets/dataset1/variants/1kg.3.annotations"
        self._createVariantAnnotationSet(vcfDir)
        self.assertTrue(self._variantSet.isAnnotated())

    def testConvertLocationHgvsC(self):
        loc = protocol.AlleleLocation()
        loc.start = 430
        loc.reference_sequence = "T"
        loc.alternate_sequence = "A"
        hgvsC = "NM_001005484.1:c.431T>A"
        testLoc = self._variantAnnotationSet.convertLocationHgvsC(hgvsC)
        self.assertEqual(testLoc, loc)

    def testConvertLocationHgvsP(self):
        loc = protocol.AlleleLocation()
        loc.start = 143
        loc.alternate_sequence = "Asn"
        loc.reference_sequence = "Ile"
        hgvsP = "NM_001005484.1:p.Ile144Asn"
        testLoc = self._variantAnnotationSet.convertLocationHgvsP(hgvsP)
        self.assertEqual(testLoc, loc)

    def testAddLocations(self):
        effect = protocol.TranscriptEffect()
        effect.hgvs_annotation.protein = "NM_001005484.1:p.Ile144Asn"
        effect.hgvs_annotation.transcript = "NM_001005484.1:c.431T>A"
        effect.protein_location.alternate_sequence = "Asn"
        effect.protein_location.reference_sequence = "Ile"
        effect.protein_location.start = 143
        effect.cds_location.alternate_sequence = "A"
        effect.cds_location.reference_sequence = "T"
        effect.cds_location.start = 430
        effect.cdna_location.start = 430
        protPos = "144/305"
        cdnaPos = "431/918"
        testEffect = self._variantAnnotationSet.addLocations(
            effect, protPos, cdnaPos)
        self.assertEqual(testEffect, effect)

    def testHashVariantAnnotation(self):
        annotation = protocol.VariantAnnotation()
        variant = protocol.Variant()
        expected = hashlib.md5('\t()\t[]\t').hexdigest()
        hashed = self._variantAnnotationSet.hashVariantAnnotation(
            variant, annotation)
        self.assertEqual(hashed, expected)

    def testGetTranscriptEffectId(self):
        effect = protocol.TranscriptEffect()
        expected = hashlib.md5("\t\t[]\t").hexdigest()
        hashed = self._variantAnnotationSet.getTranscriptEffectId(effect)
        self.assertEqual(hashed, expected)



# Copyright 2010-2016 by Haibao Tang et al. All rights reserved.
#
# This code is part of the goatools distribution and goverend by its
# license. Please see the LICENSE file included with goatools.

# This file was copied from https://github.com/tanghaibao/goatools on
# 2016-05-18. The file is unaltered other than adding this notice and
# by including the text of the LICENSE file here:

# Copyright (c) 2010-2014, Haibao Tang
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without modification,
# are permitted provided that the following conditions are met:
#
# Redistributions of source code must retain the above copyright notice, this list
# of conditions and the following disclaimer.
#
# Redistributions in binary form must reproduce the above copyright notice, this
# list of conditions and the following disclaimer in the documentation and/or
# other materials provided with the distribution.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
# ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
# DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR
# ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
# (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
# LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON
# ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
# (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
# SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

"""Read and store Gene Ontology's obo file."""
# -*- coding: UTF-8 -*-
from __future__ import print_function
from collections import defaultdict
import sys
import os
import re

GraphEngines = ("pygraphviz", "pydot")

__copyright__ = "Copyright (C) 2010-2016, H Tang et al., All rights reserved."
__author__ = "various"

class OBOReader(object):
    """Read goatools.org's obo file. Load into this iterable class.

        Download obo from: http://geneontology.org/ontology/go-basic.obo

        >>> reader = OBOReader()
        >>> for rec in reader:
                print rec
    """

    def __init__(self, obo_file="go-basic.obo", optional_attrs=None):
        """Read obo file. Load dictionary."""
        self._init_optional_attrs(optional_attrs)
        self.format_version = None
        self.data_version = None
        self.typedefs = {}

        # True if obo file exists or if a link to an obo file exists.
        if os.path.isfile(obo_file):
            self.obo_file = obo_file
            # GOTerm attributes that are necessary for any operations:
        else:
            raise Exception("download obo file first\n "
                            "[http://geneontology.org/ontology/"
                            "go-basic.obo]")

    def __iter__(self):
        """Return one GO Term record at a time from an obo file."""
        # Written by DV Klopfenstein
        # Wait to open file until needed. Automatically close file when done.
        with open(self.obo_file) as fstream:
            rec_curr = None # Stores current GO Term
            typedef_curr = None  # Stores current typedef
            for lnum, line in enumerate(fstream):
                # obo lines start with any of: [Term], [Typedef], /^\S+:/, or /^\s*/
                if self.data_version is None:
                    self._init_obo_version(line)
                if line[0:6].lower() == "[term]":
                    rec_curr = self._init_goterm_ref(rec_curr, "Term", lnum)
                elif line[0:9].lower() == "[typedef]":
                    typedef_curr = self._init_typedef(rec_curr, "Typedef", lnum)
                elif rec_curr is not None or typedef_curr is not None:
                    line = line.rstrip() # chomp
                    if ":" in line:
                        if rec_curr is not None:
                            self._add_to_ref(rec_curr, line, lnum)
                        else:
                            self._add_to_typedef(typedef_curr, line, lnum)
                    elif line == "":
                        if rec_curr is not None:
                            yield rec_curr
                            rec_curr = None
                        elif typedef_curr is not None:
                            # Save typedef.
                            self.typedefs[typedef_curr.id] = typedef_curr
                            typedef_curr = None
                    else:
                        self._die("UNEXPECTED LINE CONTENT: {L}".format(L=line), lnum)
            # Return last record, if necessary
            if rec_curr is not None:
                yield rec_curr

    def _init_obo_version(self, line):
        """Save obo version and release."""
        if line[0:14] == "format-version":
            self.format_version = line[16:-1]
        if line[0:12] == "data-version":
            self.data_version = line[14:-1]

    def _init_goterm_ref(self, rec_curr, name, lnum):
        """Initialize new reference and perform checks."""
        if rec_curr is None:
            return GOTerm()
        msg = "PREVIOUS {REC} WAS NOT TERMINATED AS EXPECTED".format(REC=name)
        self._die(msg, lnum)

    def _init_typedef(self, typedef_curr, name, lnum):
        """Initialize new typedef and perform checks."""
        if typedef_curr is None:
            return TypeDef()
        msg = "PREVIOUS {REC} WAS NOT TERMINATED AS EXPECTED".format(REC=name)
        self._die(msg, lnum)

    def _add_to_ref(self, rec_curr, line, lnum):
        """Add new fields to the current reference."""
        # Written by DV Klopfenstein
        # Examples of record lines containing ':' include:
        #   id: GO:0000002
        #   name: mitochondrial genome maintenance
        #   namespace: biological_process
        #   def: "The maintenance of ...
        #   is_a: GO:0007005 ! mitochondrion organization
        mtch = re.match(r'^(\S+):\s*(\S.*)$', line)
        if mtch:
            field_name = mtch.group(1)
            field_value = mtch.group(2)
            if field_name == "id":
                self._chk_none(rec_curr.id, lnum)
                rec_curr.id = field_value
            elif field_name == "alt_id":
                rec_curr.alt_ids.append(field_value)
            elif field_name == "name":
                self._chk_none(rec_curr.name, lnum)
                rec_curr.name = field_value
            elif field_name == "namespace":
                self._chk_none(rec_curr.namespace, lnum)
                rec_curr.namespace = field_value
            elif field_name == "is_a":
                rec_curr._parents.append(field_value.split()[0])
            elif field_name == "is_obsolete" and field_value == "true":
                rec_curr.is_obsolete = True
            elif field_name in self.optional_attrs:
                self.update_rec(rec_curr, field_name, field_value)
        else:
            self._die("UNEXPECTED FIELD CONTENT: {L}\n".format(L=line), lnum)

    def update_rec(self, rec, name, value):
        """Update current GOTerm with optional record."""
        # 'def' is a reserved word in python, do not use it as a Class attr.
        if name == "def":
            name = "defn"

        # If we have a relationship, then we will split this into a further
        # dictionary.

        if hasattr(rec, name):
            if name not in self.attrs_scalar:
                if name not in self.attrs_nested:
                    getattr(rec, name).add(value)
                else:
                    self._add_nested(rec, name, value)
            else:
                raise Exception("ATTR({NAME}) ALREADY SET({VAL})".format(
                    NAME=name, VAL=getattr(rec, name)))
        else: # Initialize new GOTerm attr
            if name in self.attrs_scalar:
                setattr(rec, name, value)
            elif name not in self.attrs_nested:
                setattr(rec, name, set([value]))
            else:
                name = '_{:s}'.format(name)
                setattr(rec, name, defaultdict(list))
                self._add_nested(rec, name, value)

    def _add_to_typedef(self, typedef_curr, line, lnum):
        """Add new fields to the current typedef."""
        mtch = re.match(r'^(\S+):\s*(\S.*)$', line)
        if mtch:
            field_name = mtch.group(1)
            field_value = mtch.group(2).split('!')[0].rstrip()

            if field_name == "id":
                self._chk_none(typedef_curr.id, lnum)
                typedef_curr.id = field_value
            elif field_name == "name":
                self._chk_none(typedef_curr.name, lnum)
                typedef_curr.name = field_value
            elif field_name == "transitive_over":
                typedef_curr.transitive_over.append(field_value)
            elif field_name == "inverse_of":
                self._chk_none(typedef_curr.inverse_of, lnum)
                typedef_curr.inverse_of = field_value
            # Note: there are other tags that aren't imported here.
        else:
            self._die("UNEXPECTED FIELD CONTENT: {L}\n".format(L=line), lnum)

    def _add_nested(self, rec, name, value):
        """Adds a term's nested attributes."""
        # Remove comments and split term into typedef / target term.
        (typedef, target_term) = value.split('!')[0].rstrip().split(' ')

        # Save the nested term.
        getattr(rec, name)[typedef].append(target_term)

    def _init_optional_attrs(self, optional_attrs):
        """Prepare to store data from user-desired optional fields.

          Not loading these optional fields by default saves in space and speed.
          But allow the possibility for saving these fields, if the user desires,
            Including:
              comment consider def is_class_level is_metadata_tag is_transitive
              relationship replaced_by subset synonym transitive_over xref
        """
        # Written by DV Klopfenstein
        # Required attributes are always loaded. All others are optionally loaded.
        self.attrs_req = ['id', 'alt_id', 'name', 'namespace', 'is_a', 'is_obsolete']
        self.attrs_scalar = ['comment', 'defn',
                             'is_class_level', 'is_metadata_tag',
                             'is_transitive', 'transitive_over']
        self.attrs_nested = frozenset(['relationship'])
        # Allow user to specify either: 'def' or 'defn'
        #   'def' is an obo field name, but 'defn' is legal Python attribute name
        fnc = lambda aopt: aopt if aopt != "defn" else "def"
        if optional_attrs is None:
            optional_attrs = []
        elif isinstance(optional_attrs, str):
            optional_attrs = [fnc(optional_attrs)] if optional_attrs not in self.attrs_req else []
        elif isinstance(optional_attrs, list) or isinstance(optional_attrs, set):
            optional_attrs = set([fnc(f) for f in optional_attrs if f not in self.attrs_req])
        else:
            raise Exception("optional_attrs arg MUST BE A str, list, or set.")
        self.optional_attrs = optional_attrs


    def _die(self, msg, lnum):
        """Raise an Exception if file read is unexpected."""
        raise Exception("**FATAL {FILE}({LNUM}): {MSG}\n".format(
            FILE=self.obo_file, LNUM=lnum, MSG=msg))

    def _chk_none(self, init_val, lnum):
        """Expect these lines to be uninitialized."""
        if init_val is None or init_val is "":
            return
        self._die("FIELD IS ALREADY INITIALIZED", lnum)




class GOTerm:
    """
    GO term, actually contain a lot more properties than interfaced here
    """

    def __init__(self):
        self.id = ""                # GO:NNNNNNN
        self.name = ""              # description
        self.namespace = ""         # BP, CC, MF
        self._parents = []          # is_a basestring of parents
        self.parents = []           # parent records
        self.children = []          # children records
        self.level = None           # shortest distance from root node
        self.depth = None           # longest distance from root node
        self.is_obsolete = False    # is_obsolete
        self.alt_ids = []           # alternative identifiers

    def __str__(self):
        obsolete = "obsolete" if self.is_obsolete else ""
        return "{GO}\tlevel-{L:>02}\tdepth-{D:>02}\t{NAME} [{NS}] {OLD}".format(
            GO=self.id, L=self.level, D=self.depth,
            NAME=self.name, NS=self.namespace, OLD=obsolete)

    def __repr__(self):
        """Print GO id and all attributes in GOTerm class."""
        ret = ["GOTerm('{ID}'):".format(ID=self.id)]
        for key, val in self.__dict__.items():
            if isinstance(val, int) or isinstance(val, str):
                ret.append("{K}:{V}".format(K=key, V=val))
            else:
                ret.append("{K}: {V} items".format(K=key, V=len(val)))
                if len(val) < 10:
                    if not isinstance(val, dict):
                        for elem in val:
                            ret.append("  {ELEM}".format(ELEM=elem))
                    else:
                        for (typedef, terms) in val.items():
                            ret.append("  {TYPEDEF}: {NTERMS} items"
                                       .format(TYPEDEF=typedef,
                                               NTERMS=len(terms)))
                            for t in terms:
                                ret.append("    {TERM}".format(TERM=t))
        return "\n  ".join(ret)

    def has_parent(self, term):
        for p in self.parents:
            if p.id == term or p.has_parent(term):
                return True
        return False

    def has_child(self, term):
        for p in self.children:
            if p.id == term or p.has_child(term):
                return True
        return False

    def get_all_parents(self):
        all_parents = set()
        for p in self.parents:
            all_parents.add(p.id)
            all_parents |= p.get_all_parents()
        return all_parents

    def get_all_children(self):
        all_children = set()
        for p in self.children:
            all_children.add(p.id)
            all_children |= p.get_all_children()
        return all_children

    def get_all_parent_edges(self):
        all_parent_edges = set()
        for p in self.parents:
            all_parent_edges.add((self.id, p.id))
            all_parent_edges |= p.get_all_parent_edges()
        return all_parent_edges

    def get_all_child_edges(self):
        all_child_edges = set()
        for p in self.children:
            all_child_edges.add((p.id, self.id))
            all_child_edges |= p.get_all_child_edges()
        return all_child_edges

    def write_hier_rec(self, gos_printed, out=sys.stdout,
                      len_dash=1, max_depth=None, num_child=None, short_prt=False,
                      include_only=None, go_marks=None,
                      depth=1, dp="-"):
        """Write hierarchy for a GO Term record."""
        # Added by DV Klopfenstein
        GO_id = self.id
        # Shortens hierarchy report by only printing the hierarchy
        # for the sub-set of user-specified GO terms which are connected.
        if include_only is not None and GO_id not in include_only:
          return
        nrp = short_prt and GO_id in gos_printed
        if go_marks is not None:
          out.write('{} '.format('>' if GO_id in go_marks else ' '))
        if len_dash is not None:
            # Default character indicating hierarchy level is '-'.
            # '=' is used to indicate a hierarchical path printed in detail previously.
            letter = '-' if not nrp or not self.children else '='
            dp = ''.join([letter]*depth)
            out.write('{DASHES:{N}} '.format(DASHES=dp, N=len_dash))
        if num_child is not None:
            out.write('{N:>5} '.format(N=len(self.get_all_children())))
        out.write('{GO}\tL-{L:>02}\tD-{D:>02}\t{desc}\n'.format(
            GO=self.id, L=self.level, D=self.depth, desc=self.name))
        # Track GOs previously printed only if needed
        if short_prt:
          gos_printed.add(GO_id)
        # Do not print hierarchy below this turn if it has already been printed
        if nrp:
            return
        depth += 1
        if max_depth is not None and depth > max_depth:
            return
        for p in self.children:
            p.write_hier_rec(gos_printed, out, len_dash, max_depth, num_child, short_prt,
                include_only, go_marks,
                depth, dp)


class TypeDef(object):
    """
        TypeDef term. These contain more tags than included here, but these
        are the most important.
    """

    def __init__(self):
        self.id = ""                # GO:NNNNNNN
        self.name = ""              # description
        self.transitive_over = []   # List of other typedefs
        self.inverse_of = ""        # Name of inverse typedef.

    def __str__(self):
        ret = []
        ret.append("Typedef - {} ({}):".format(self.id, self.name))
        ret.append("  Inverse of: {}".format(self.inverse_of
                                             if self.inverse_of else "None"))
        if self.transitive_over:
            ret.append("  Transitive over:")
            for t in self.transitive_over:
                ret.append("    - {}".format(t))
        return "\n".join(ret)


class GODag(dict):

    def __init__(self, obo_file="go-basic.obo", optional_attrs=None):
        self.load_obo_file(obo_file, optional_attrs)

    def load_obo_file(self, obo_file, optional_attrs):

        print("load obo file %s" % obo_file, file=sys.stderr)
        reader = OBOReader(obo_file, optional_attrs)
        for rec in reader:
            self[rec.id] = rec
            for alt in rec.alt_ids:
                self[alt] = rec

        print("{OBO}: format-version({FMT}) data-version({REL})".format(
            OBO=obo_file, FMT=reader.format_version, REL=reader.data_version))

        # Save the typedefs and parsed optional_attrs
        self.typedefs = reader.typedefs
        self.optional_attrs = reader.optional_attrs

        self.populate_terms()
        print(len(self), "nodes imported", file=sys.stderr)

    def populate_terms(self):

        def _init_level(rec):
            if rec.level is None:
                if not rec.parents:
                    rec.level = 0
                else:
                    rec.level = min(_init_level(rec) for rec in rec.parents) + 1
            return rec.level

        def _init_depth(rec):
            if rec.depth is None:
                if not rec.parents:
                    rec.depth = 0
                else:
                    rec.depth = max(_init_depth(rec) for rec in rec.parents) + 1
            return rec.depth

        # Make parents and relationships references to the actual GO terms.
        for rec in self.values():
            rec.parents = [self[x] for x in rec._parents]

            if hasattr(rec, '_relationship'):
                rec.relationship = defaultdict(set)
                for (typedef, terms) in rec._relationship.items():
                    rec.relationship[typedef].update(set([self[x] for x in terms]))
                delattr(rec, '_relationship')

        # populate children, levels and add inverted relationships
        for rec in self.values():
            for p in rec.parents:
                if rec not in p.children:
                    p.children.append(rec)

            # Add invert relationships
            if hasattr(rec, 'relationship'):
                for (typedef, terms) in rec.relationship.items():
                    invert_typedef = self.typedefs[typedef].inverse_of
                    if invert_typedef:
                        # Add inverted relationship
                        for t in terms:
                            if not hasattr(t, 'relationship'):
                                t.relationship = defaultdict(set)
                            t.relationship[invert_typedef].add(rec)

            if rec.level is None:
                _init_level(rec)

            if rec.depth is None:
                _init_depth(rec)

    def write_dag(self, out=sys.stdout):
        """Write info for all GO Terms in obo file, sorted numerically."""
        for rec_id, rec in sorted(self.items()):
            print(rec, file=out)

    def write_hier_all(self, out=sys.stdout,
                      len_dash=1, max_depth=None, num_child=None, short_prt=False):
        """Write hierarchy for all GO Terms in obo file."""
        # Print: [biological_process, molecular_function, and cellular_component]
        for go_id in ['GO:0008150', 'GO:0003674', 'GO:0005575']:
          self.write_hier(go_id, out, len_dash, max_depth, num_child, short_prt, None)

    def write_hier(self, GO_id, out=sys.stdout,
                       len_dash=1, max_depth=None, num_child=None, short_prt=False,
                       include_only=None, go_marks=None):
        """Write hierarchy for a GO Term."""
        gos_printed = set()
        self[GO_id].write_hier_rec(gos_printed, out, len_dash, max_depth, num_child,
            short_prt, include_only, go_marks)

    @staticmethod
    def id2int(GO_id): return int(GO_id.replace("GO:", "", 1))

    def query_term(self, term, verbose=False):
        if term not in self:
            print("Term %s not found!" % term, file=sys.stderr)
            return

        rec = self[term]
        if verbose:
            print(rec)
            print("all parents:", rec.get_all_parents(), file=sys.stderr)
            print("all children:", rec.get_all_children(), file=sys.stderr)
        return rec

    def paths_to_top(self, term):
        """ Returns all possible paths to the root node

            Each path includes the term given. The order of the path is
            top -> bottom, i.e. it starts with the root and ends with the
            given term (inclusively).

            Parameters:
            -----------
            - term:
                the id of the GO term, where the paths begin (i.e. the
                accession 'GO:0003682')

            Returns:
            --------
            - a list of lists of GO Terms
        """
        # error handling consistent with original authors
        if term not in self:
            print("Term %s not found!" % term, file=sys.stderr)
            return

        def _paths_to_top_recursive(rec):
            if rec.level == 0:
                return [[rec]]
            paths = []
            for parent in rec.parents:
                top_paths = _paths_to_top_recursive(parent)
                for top_path in top_paths:
                    top_path.append(rec)
                    paths.append(top_path)
            return paths

        go_term = self[term]
        return _paths_to_top_recursive(go_term)

    def _label_wrap(self, label):
        wrapped_label = r"%s\n%s" % (label,
                                     self[label].name.replace(",", r"\n"))
        return wrapped_label

    def make_graph_pydot(self, recs, nodecolor,
                     edgecolor, dpi,
                     draw_parents=True, draw_children=True):
        """draw AMIGO style network, lineage containing one query record."""
        import pydot
        G = pydot.Dot(graph_type='digraph', dpi="{}".format(dpi)) # Directed Graph
        edgeset = set()
        usr_ids = [rec.id for rec in recs]
        for rec in recs:
            if draw_parents:
                edgeset.update(rec.get_all_parent_edges())
            if draw_children:
                edgeset.update(rec.get_all_child_edges())

        lw = self._label_wrap
        rec_id_set = set([rec_id for endpts in edgeset for rec_id in endpts])
        nodes = {str(ID):pydot.Node(
              lw(ID).replace("GO:",""),  # Node name
              shape="box",
              style="rounded, filled",
              # Highlight query terms in plum:
              fillcolor="beige" if ID not in usr_ids else "plum",
              color=nodecolor)
                for ID in rec_id_set}

        # add nodes explicitly via add_node
        for rec_id, node in nodes.items():
            G.add_node(node)

        for src, target in edgeset:
            # default layout in graphviz is top->bottom, so we invert
            # the direction and plot using dir="back"
            G.add_edge(pydot.Edge(nodes[target], nodes[src],
              shape="normal",
              color=edgecolor,
              label="is_a",
              dir="back"))

        return G

    def make_graph_pygraphviz(self, recs, nodecolor,
                     edgecolor, dpi,
                     draw_parents=True, draw_children=True):
        # draw AMIGO style network, lineage containing one query record
        import pygraphviz as pgv

        G = pgv.AGraph(name="GO tree")
        edgeset = set()
        for rec in recs:
            if draw_parents:
                edgeset.update(rec.get_all_parent_edges())
            if draw_children:
                edgeset.update(rec.get_all_child_edges())

        edgeset = [(self._label_wrap(a), self._label_wrap(b))
                   for (a, b) in edgeset]

        # add nodes explicitly via add_node
        # adding nodes implicitly via add_edge misses nodes
        # without at least one edge
        for rec in recs:
            G.add_node(self._label_wrap(rec.id))

        for src, target in edgeset:
            # default layout in graphviz is top->bottom, so we invert
            # the direction and plot using dir="back"
            G.add_edge(target, src)

        G.graph_attr.update(dpi="%d" % dpi)
        G.node_attr.update(shape="box", style="rounded,filled",
                           fillcolor="beige", color=nodecolor)
        G.edge_attr.update(shape="normal", color=edgecolor,
                           dir="back", label="is_a")
        # highlight the query terms
        for rec in recs:
            try:
                q = G.get_node(self._label_wrap(rec.id))
                q.attr.update(fillcolor="plum")
            except:
                continue

        return G

    def draw_lineage(self, recs, nodecolor="mediumseagreen",
                     edgecolor="lightslateblue", dpi=96,
                     lineage_img="GO_lineage.png", engine="pygraphviz",
                     gml=False, draw_parents=True, draw_children=True):
        assert engine in GraphEngines
        if engine == "pygraphviz":
            G = self.make_graph_pygraphviz(recs, nodecolor, edgecolor, dpi,
                              draw_parents=draw_parents, draw_children=draw_children)
        else:
            G = self.make_graph_pydot(recs, nodecolor, edgecolor, dpi,
                              draw_parents=draw_parents, draw_children=draw_children)

        if gml:
            import networkx as nx  # use networkx to do the conversion
            pf = lineage_img.rsplit(".", 1)[0]
            NG = nx.from_agraph(G) if engine == "pygraphviz" else nx.from_pydot(G)

            del NG.graph['node']
            del NG.graph['edge']
            gmlfile = pf + ".gml"
            nx.write_gml(NG, gmlfile)
            print("GML graph written to {0}".format(gmlfile), file=sys.stderr)

        print(("lineage info for terms %s written to %s" %
                             ([rec.id for rec in recs], lineage_img)), file=sys.stderr)

        if engine == "pygraphviz":
            G.draw(lineage_img, prog="dot")
        else:
            G.write_png(lineage_img)

    def update_association(self, association):
        bad_terms = set()
        for key, terms in list(association.items()):
            parents = set()
            for term in terms:
                try:
                    parents.update(self[term].get_all_parents())
                except:
                    bad_terms.add(term.strip())
            terms.update(parents)
        if bad_terms:
            print("terms not found: %s" % (bad_terms,), file=sys.stderr)

# Copyright (C) 2010-2016, H Tang et al., All rights reserved.



"""
Data-driven tests for variants.
"""
from __future__ import division
from __future__ import print_function
from __future__ import unicode_literals

import os
import glob

import vcf

import ga4gh.datamodel as datamodel
import ga4gh.datamodel.datasets as datasets
import ga4gh.datamodel.variants as variants
import ga4gh.datamodel.references as references
import ga4gh.datamodel.ontologies as ontologies
import ga4gh.protocol as protocol
import tests.datadriven as datadriven
import tests.paths as paths


def testVariantAnnotationSets():
    testDataDir = "tests/data/datasets/dataset1/variants"
    for test in datadriven.makeTests(testDataDir, VariantAnnotationSetTest):
        yield test


class VariantAnnotationSetTest(datadriven.DataDrivenTest):
    """
    Data driven test class for variant annotation sets. Builds an
    alternative model of a variant annotation set using pyvcf, and
    verifies that it is consistent with the model built by the
    variants.VariantAnnotationSet object.
    """
    def __init__(self, variantAnnotationSetId, baseDir):
        super(VariantAnnotationSetTest, self).__init__(
            variantAnnotationSetId, baseDir)
        self._variantRecords = []
        self._referenceNames = set()
        # Only read in VCF files that are annotated.
        for vcfFile in glob.glob(os.path.join(self._dataPath, "*.vcf.gz")):
            if self._isAnnotated():
                self._readVcf(vcfFile)
        self._isCsq = self._hasConsequenceField()

    def _isAnnotated(self):
        """
        Determines whether the variant set under test is annotated.

        :return: Boolean
        """
        pyvcfreader = vcf.Reader(
            filename=glob.glob(
                os.path.join(self._dataPath, "*.vcf.gz"))[0])
        items = [x for x in pyvcfreader.infos]
        return ('ANN' in items) or ('CSQ' in items)

    def _hasConsequenceField(self):
        pyvcfreader = vcf.Reader(
            filename=glob.glob(
                os.path.join(self._dataPath, "*.vcf.gz"))[0])
        items = [x for x in pyvcfreader.infos]
        return 'CSQ' in items

    def _readVcf(self, vcfFileName):
        """
        Reads all variants and metadata from the specified VCF file and
        store locally.
        """
        vcfReader = vcf.Reader(filename=vcfFileName)
        metadata = vcfReader.metadata
        self._vcfVersion = metadata["fileformat"]
        self._infos = vcfReader.infos
        self._formats = vcfReader.formats
        self.vcfSamples = vcfReader.samples
        if "VEP" in metadata:
            self._isVEP = True
        else:
            self._isVEP = False
        for record in vcfReader:
            self._referenceNames.add(record.CHROM)
            # When an END info tag is present it takes precedence
            if "END" in record.INFO:
                record.end = record.INFO["END"]
            self._variantRecords.append(record)

    def splitAnnField(self, annfield):
        """
        Splits the `ANN=` record of a VCF into a dictionary of
        key, value. This is brittle but covers the cases of the

        :param annfield:
        :return: dictionary of k, v pairs along from the ann string
        """
        # Have yet to see if other files follow this pattern
        if self._isVEP and not self._isCsq:
            fields = ["alt", "effects", "impact", "symbol", "geneName",
                      "featureType", "featureId", "trBiotype", "exon",
                      "intron", "hgvsC", "hgvsP",
                      "cdnaPos", "cdsPos", "protPos",
                      "aminos", "codons", "existingVar", "distance",
                      "strand", "symbolSource", "hgncId", "hgvsOffset"]
        else:
            fields = ["alt", "effects", "impact", "geneName",
                      "geneId", "featureType",
                      "featureId", "trBiotype", "rank", "hgvsC",
                      "hgvsP", "cdnaPos", "cdsPos", "protPos",
                      "distance", "errsWarns"]
        values = annfield.split('|')
        return dict(zip(fields, values))

    def getDataModelInstance(self, localId, dataPath):
        dataset = datasets.Dataset("ds")
        variantSet = variants.HtslibVariantSet(dataset, localId)
        variantSet.populateFromDirectory(dataPath)
        referenceSet = references.AbstractReferenceSet("rs")
        variantSet.setReferenceSet(referenceSet)
        if variantSet.isAnnotated():
            sequenceOntology = ontologies.Ontology(paths.ontologyName)
            sequenceOntology.populateFromFile(paths.ontologyPath)
            annotationSet = variantSet.getVariantAnnotationSets()[0]
            annotationSet.setOntology(sequenceOntology)
            return annotationSet
        else:
            return variantSet

    def getProtocolClass(self):
        if self._isAnnotated():
            return protocol.VariantAnnotationSet
        else:
            return protocol.VariantSet

    def _compareTwoListFloats(self, a, b):
        for ai, bi in zip(a, b):
            self.assertAlmostEqual(float(ai), float(bi), 5)

    def _verifyVariantAnnotationsEqual(
            self, gaVariantAnnotations, pyvcfVariants):
        """
        Verifies that the lists of GA4GH variants and pyvcf variants
        are equivalent.
        """
        self.assertEqual(len(gaVariantAnnotations), len(pyvcfVariants))
        for pair, pyvcfVariant in zip(gaVariantAnnotations, pyvcfVariants):
            gaVariant, gaVariantAnnotation = pair
            # TODO parse and compare with analysis.info
            # pyvcfInfo = pyvcfVariant.INFO
            # pyvcf uses 1-based indexing.
            self.assertEqual(gaVariant.id, gaVariantAnnotation.variant_id)
            self.assertEqual(gaVariant.start, pyvcfVariant.POS - 1)
            self.assertEqual(gaVariant.end, pyvcfVariant.end)
            # Annotated VCFs contain an ANN field in the record info
            if 'ANN' in pyvcfVariant.INFO:
                pyvcfAnn = pyvcfVariant.INFO['ANN']
                i = 0
                for pyvcfEffect, gaEffect in \
                        zip(pyvcfAnn, gaVariantAnnotation.transcript_effects):
                    effectDict = self.splitAnnField(pyvcfEffect)
                    self.assertEqual(
                        gaEffect.alternate_bases, effectDict['alt'])
                    self.assertEqual(
                        gaEffect.feature_id, effectDict['featureId'])
                    self.assertEqual(
                        gaEffect.hgvs_annotation.transcript,
                        effectDict['hgvsC'])
                    self.assertEqual(
                        gaEffect.hgvs_annotation.protein, effectDict['hgvsP'])
                    if 'HGVS.g' in pyvcfVariant.INFO:
                        # Not all VCF have this field
                        index = i % len(pyvcfVariant.INFO['HGVS.g'])
                        self.assertEqual(
                            gaEffect.hgvs_annotation.genomic,
                            pyvcfVariant.INFO['HGVS.g'][index])
                    i += 1
            elif 'CSQ' in pyvcfVariant.INFO:
                pyvcfAnn = pyvcfVariant.INFO['CSQ']
                transcriptEffects = []
                for ann in pyvcfAnn:
                    transcriptEffects += self._splitCsqEffects(ann)
                for treff, gaEffect in zip(
                        transcriptEffects,
                        gaVariantAnnotation.transcript_effects):
                    self.assertEqual(gaEffect.alternate_bases, treff['alt'])
                    self.assertEqual(gaEffect.feature_id, treff['featureId'])
            self.assertGreater(len(gaVariantAnnotation.transcript_effects), 0)

    def _splitCsqEffects(self, annStr):
        (alt, gene, featureId, featureType, effects, cdnaPos,
            cdsPos, protPos, aminos, codons, existingVar,
            distance, strand, sift, polyPhen, motifName,
            motifPos, highInfPos,
            motifScoreChange) = annStr.split('|')
        terms = effects.split("&")
        treffs = []
        for term in terms:
            effect = {}
            effect['featureId'] = featureId
            effect['alt'] = alt
            treffs.append(effect)
        return treffs

    def testVariantsValid(self):
        end = datamodel.PysamDatamodelMixin.vcfMax
        for referenceName in self._referenceNames:
            iterator = self._gaObject.getVariantAnnotations(
                referenceName, 0, end)
            for gaVariant, gaVariantAnnotation in iterator:
                self.assertValid(protocol.VariantAnnotation,
                                 protocol.toJson(gaVariantAnnotation))

    def _getPyvcfVariants(
            self, referenceName, startPosition=0, endPosition=2**30):
        """
        variants with in interval [startPosition, endPosition)
        """
        localVariants = []
        for variant in self._variantRecords:
            case1 = (variant.start <= startPosition and
                     variant.end > startPosition)
            case2 = (variant.start > startPosition and
                     variant.start < endPosition)
            if (variant.CHROM == referenceName) and (case1 or case2):
                localVariants.append(variant)
        return localVariants

    def _assertEmptyVariant(self, referenceName, startPosition, endPosition):
        gaVariantAnnotations = list(self._gaObject.getVariantAnnotations(
            referenceName, startPosition, endPosition))
        self.assertEqual(len(gaVariantAnnotations), 0)
        pyvcfVariants = self._getPyvcfVariants(
            referenceName, startPosition, endPosition)
        self.assertEqual(len(pyvcfVariants), 0)

    def _assertVariantsEqualInRange(
            self, referenceName, startPosition, endPosition):
        gaVariantAnnotations = list(
            self._gaObject.getVariantAnnotations(
                referenceName, startPosition, endPosition))
        pyvcfVariants = self._getPyvcfVariants(
            referenceName, startPosition, endPosition)
        self.assertGreaterEqual(len(gaVariantAnnotations), 0)
        self._verifyVariantAnnotationsEqual(
            gaVariantAnnotations, pyvcfVariants)

    def testVariantAnnotationInSegments(self):
        for referenceName in self._referenceNames:
            localVariants = self._getPyvcfVariants(referenceName)
            mini = localVariants[0].start
            # NOTE, the end of the last variant may not reflect the END
            maxi = max([v.end for v in localVariants])
            seglen = (maxi-mini) // 3
            seg1 = mini + seglen
            seg2 = seg1 + seglen
            self._assertEmptyVariant(referenceName, -1, mini)
            self._assertEmptyVariant(referenceName, maxi, maxi+100)
            self._assertVariantsEqualInRange(referenceName, mini, seg1)
            self._assertVariantsEqualInRange(referenceName, seg1, seg2)
            self._assertVariantsEqualInRange(referenceName, seg2, maxi)

    def _gaVariantAnnotationEqualsPyvcfVariantAnnotation(
            self, gaVariant, gaAnnotation, pyvcfVariant):
        if gaVariant.start != pyvcfVariant.start:
            return False
        if gaVariant.end != pyvcfVariant.end:
            return False
        return True

    def _pyvcfVariantAnnotationIsInGaVariantAnnotations(
            self, pyvcfVariant, intervalStart, intervalEnd):
        isIn = False
        pairs = list(self._gaObject.getVariantAnnotations(
            pyvcfVariant.CHROM, intervalStart, intervalEnd))
        for gaVariant, gaAnnotation in pairs:
            if self._gaVariantAnnotationEqualsPyvcfVariantAnnotation(
                    gaVariant, gaAnnotation, pyvcfVariant):
                isIn = True
                break
        return isIn

    def testVariantAnnotationFromToEveryVariantAnnotation(self):
        for variant in self._variantRecords:
            variantStart = variant.start
            variantEnd = variant.end
            # Cases of search interval does not overlap with variant
            # interval on the left

            self.assertFalse(
                self._pyvcfVariantAnnotationIsInGaVariantAnnotations(
                    variant, variantStart-2, variantStart-1))
            self.assertFalse(
                self._pyvcfVariantAnnotationIsInGaVariantAnnotations(
                    variant, variantStart-1, variantStart))
            # interval on the right
            self.assertFalse(
                self._pyvcfVariantAnnotationIsInGaVariantAnnotations(
                    variant, variantEnd, variantEnd+1))
            self.assertFalse(
                self._pyvcfVariantAnnotationIsInGaVariantAnnotations(
                    variant, variantEnd+1, variantEnd+2))
            # case of search interval is within variant

            self.assertTrue(
                self._pyvcfVariantAnnotationIsInGaVariantAnnotations(
                    variant, variantStart, variantEnd))
            if (variantEnd - variantStart) != 1:
                self.assertTrue(
                    self._pyvcfVariantAnnotationIsInGaVariantAnnotations(
                        variant, variantStart, variantEnd-1))
                self.assertTrue(
                    self._pyvcfVariantAnnotationIsInGaVariantAnnotations(
                        variant, variantStart+1, variantEnd))

            # case of search interval contains variant
            self.assertTrue(
                self._pyvcfVariantAnnotationIsInGaVariantAnnotations(
                    variant, variantStart-1, variantEnd+1))
            # cases of search interval intersec with variant
            self.assertTrue(
                self._pyvcfVariantAnnotationIsInGaVariantAnnotations(
                    variant, variantStart-1, variantStart+1))
            self.assertTrue(
                self._pyvcfVariantAnnotationIsInGaVariantAnnotations(
                    variant, variantStart, variantStart+1))
            self.assertTrue(
                self._pyvcfVariantAnnotationIsInGaVariantAnnotations(
                    variant, variantEnd-1, variantEnd))
            self.assertTrue(
                self._pyvcfVariantAnnotationIsInGaVariantAnnotations(
                    variant, variantEnd-1, variantEnd+1))



# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: ga4gh/bio_metadata_service.proto

import sys
_b=sys.version_info[0]<3 and (lambda x:x) or (lambda x:x.encode('latin1'))
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
from google.protobuf import descriptor_pb2
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from ga4gh import bio_metadata_pb2 as ga4gh_dot_bio__metadata__pb2


DESCRIPTOR = _descriptor.FileDescriptor(
  name='ga4gh/bio_metadata_service.proto',
  package='ga4gh',
  syntax='proto3',
  serialized_pb=_b('\n ga4gh/bio_metadata_service.proto\x12\x05ga4gh\x1a\x18ga4gh/bio_metadata.proto\"c\n\x18SearchIndividualsRequest\x12\x12\n\ndataset_id\x18\x01 \x01(\t\x12\x0c\n\x04name\x18\x02 \x01(\t\x12\x11\n\tpage_size\x18\x03 \x01(\x05\x12\x12\n\npage_token\x18\x04 \x01(\t\"-\n\x14GetIndividualRequest\x12\x15\n\rindividual_id\x18\x01 \x01(\t\",\n\x13GetBioSampleRequest\x12\x15\n\rbio_sample_id\x18\x01 \x01(\t\"\\\n\x19SearchIndividualsResponse\x12&\n\x0bindividuals\x18\x01 \x03(\x0b\x32\x11.ga4gh.Individual\x12\x17\n\x0fnext_page_token\x18\x02 \x01(\t\"y\n\x17SearchBioSamplesRequest\x12\x12\n\ndataset_id\x18\x01 \x01(\t\x12\x0c\n\x04name\x18\x02 \x01(\t\x12\x15\n\rindividual_id\x18\x03 \x01(\t\x12\x11\n\tpage_size\x18\x04 \x01(\x05\x12\x12\n\npage_token\x18\x05 \x01(\t\"Y\n\x18SearchBioSamplesResponse\x12$\n\nbiosamples\x18\x01 \x03(\x0b\x32\x10.ga4gh.BioSample\x12\x17\n\x0fnext_page_token\x18\x02 \x01(\t2\xc0\x02\n\x12\x42ioMetadataService\x12V\n\x11SearchIndividuals\x12\x1f.ga4gh.SearchIndividualsRequest\x1a .ga4gh.SearchIndividualsResponse\x12S\n\x10SearchBioSamples\x12\x1e.ga4gh.SearchBioSamplesRequest\x1a\x1f.ga4gh.SearchBioSamplesResponse\x12?\n\rGetIndividual\x12\x1b.ga4gh.GetIndividualRequest\x1a\x11.ga4gh.Individual\x12<\n\x0cGetBioSample\x12\x1a.ga4gh.GetBioSampleRequest\x1a\x10.ga4gh.BioSampleb\x06proto3')
  ,
  dependencies=[ga4gh_dot_bio__metadata__pb2.DESCRIPTOR,])
_sym_db.RegisterFileDescriptor(DESCRIPTOR)




_SEARCHINDIVIDUALSREQUEST = _descriptor.Descriptor(
  name='SearchIndividualsRequest',
  full_name='ga4gh.SearchIndividualsRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='dataset_id', full_name='ga4gh.SearchIndividualsRequest.dataset_id', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='name', full_name='ga4gh.SearchIndividualsRequest.name', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='page_size', full_name='ga4gh.SearchIndividualsRequest.page_size', index=2,
      number=3, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='page_token', full_name='ga4gh.SearchIndividualsRequest.page_token', index=3,
      number=4, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=69,
  serialized_end=168,
)


_GETINDIVIDUALREQUEST = _descriptor.Descriptor(
  name='GetIndividualRequest',
  full_name='ga4gh.GetIndividualRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='individual_id', full_name='ga4gh.GetIndividualRequest.individual_id', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=170,
  serialized_end=215,
)


_GETBIOSAMPLEREQUEST = _descriptor.Descriptor(
  name='GetBioSampleRequest',
  full_name='ga4gh.GetBioSampleRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='bio_sample_id', full_name='ga4gh.GetBioSampleRequest.bio_sample_id', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=217,
  serialized_end=261,
)


_SEARCHINDIVIDUALSRESPONSE = _descriptor.Descriptor(
  name='SearchIndividualsResponse',
  full_name='ga4gh.SearchIndividualsResponse',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='individuals', full_name='ga4gh.SearchIndividualsResponse.individuals', index=0,
      number=1, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='next_page_token', full_name='ga4gh.SearchIndividualsResponse.next_page_token', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=263,
  serialized_end=355,
)


_SEARCHBIOSAMPLESREQUEST = _descriptor.Descriptor(
  name='SearchBioSamplesRequest',
  full_name='ga4gh.SearchBioSamplesRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='dataset_id', full_name='ga4gh.SearchBioSamplesRequest.dataset_id', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='name', full_name='ga4gh.SearchBioSamplesRequest.name', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='individual_id', full_name='ga4gh.SearchBioSamplesRequest.individual_id', index=2,
      number=3, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='page_size', full_name='ga4gh.SearchBioSamplesRequest.page_size', index=3,
      number=4, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='page_token', full_name='ga4gh.SearchBioSamplesRequest.page_token', index=4,
      number=5, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=357,
  serialized_end=478,
)


_SEARCHBIOSAMPLESRESPONSE = _descriptor.Descriptor(
  name='SearchBioSamplesResponse',
  full_name='ga4gh.SearchBioSamplesResponse',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='biosamples', full_name='ga4gh.SearchBioSamplesResponse.biosamples', index=0,
      number=1, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='next_page_token', full_name='ga4gh.SearchBioSamplesResponse.next_page_token', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=480,
  serialized_end=569,
)

_SEARCHINDIVIDUALSRESPONSE.fields_by_name['individuals'].message_type = ga4gh_dot_bio__metadata__pb2._INDIVIDUAL
_SEARCHBIOSAMPLESRESPONSE.fields_by_name['biosamples'].message_type = ga4gh_dot_bio__metadata__pb2._BIOSAMPLE
DESCRIPTOR.message_types_by_name['SearchIndividualsRequest'] = _SEARCHINDIVIDUALSREQUEST
DESCRIPTOR.message_types_by_name['GetIndividualRequest'] = _GETINDIVIDUALREQUEST
DESCRIPTOR.message_types_by_name['GetBioSampleRequest'] = _GETBIOSAMPLEREQUEST
DESCRIPTOR.message_types_by_name['SearchIndividualsResponse'] = _SEARCHINDIVIDUALSRESPONSE
DESCRIPTOR.message_types_by_name['SearchBioSamplesRequest'] = _SEARCHBIOSAMPLESREQUEST
DESCRIPTOR.message_types_by_name['SearchBioSamplesResponse'] = _SEARCHBIOSAMPLESRESPONSE

SearchIndividualsRequest = _reflection.GeneratedProtocolMessageType('SearchIndividualsRequest', (_message.Message,), dict(
  DESCRIPTOR = _SEARCHINDIVIDUALSREQUEST,
  __module__ = 'ga4gh.bio_metadata_service_pb2'
  # @@protoc_insertion_point(class_scope:ga4gh.SearchIndividualsRequest)
  ))
_sym_db.RegisterMessage(SearchIndividualsRequest)

GetIndividualRequest = _reflection.GeneratedProtocolMessageType('GetIndividualRequest', (_message.Message,), dict(
  DESCRIPTOR = _GETINDIVIDUALREQUEST,
  __module__ = 'ga4gh.bio_metadata_service_pb2'
  # @@protoc_insertion_point(class_scope:ga4gh.GetIndividualRequest)
  ))
_sym_db.RegisterMessage(GetIndividualRequest)

GetBioSampleRequest = _reflection.GeneratedProtocolMessageType('GetBioSampleRequest', (_message.Message,), dict(
  DESCRIPTOR = _GETBIOSAMPLEREQUEST,
  __module__ = 'ga4gh.bio_metadata_service_pb2'
  # @@protoc_insertion_point(class_scope:ga4gh.GetBioSampleRequest)
  ))
_sym_db.RegisterMessage(GetBioSampleRequest)

SearchIndividualsResponse = _reflection.GeneratedProtocolMessageType('SearchIndividualsResponse', (_message.Message,), dict(
  DESCRIPTOR = _SEARCHINDIVIDUALSRESPONSE,
  __module__ = 'ga4gh.bio_metadata_service_pb2'
  # @@protoc_insertion_point(class_scope:ga4gh.SearchIndividualsResponse)
  ))
_sym_db.RegisterMessage(SearchIndividualsResponse)

SearchBioSamplesRequest = _reflection.GeneratedProtocolMessageType('SearchBioSamplesRequest', (_message.Message,), dict(
  DESCRIPTOR = _SEARCHBIOSAMPLESREQUEST,
  __module__ = 'ga4gh.bio_metadata_service_pb2'
  # @@protoc_insertion_point(class_scope:ga4gh.SearchBioSamplesRequest)
  ))
_sym_db.RegisterMessage(SearchBioSamplesRequest)

SearchBioSamplesResponse = _reflection.GeneratedProtocolMessageType('SearchBioSamplesResponse', (_message.Message,), dict(
  DESCRIPTOR = _SEARCHBIOSAMPLESRESPONSE,
  __module__ = 'ga4gh.bio_metadata_service_pb2'
  # @@protoc_insertion_point(class_scope:ga4gh.SearchBioSamplesResponse)
  ))
_sym_db.RegisterMessage(SearchBioSamplesResponse)


# @@protoc_insertion_point(module_scope)



# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: ga4gh/reference_service.proto

import sys
_b=sys.version_info[0]<3 and (lambda x:x) or (lambda x:x.encode('latin1'))
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
from google.protobuf import descriptor_pb2
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from ga4gh import references_pb2 as ga4gh_dot_references__pb2


DESCRIPTOR = _descriptor.FileDescriptor(
  name='ga4gh/reference_service.proto',
  package='ga4gh',
  syntax='proto3',
  serialized_pb=_b('\n\x1dga4gh/reference_service.proto\x12\x05ga4gh\x1a\x16ga4gh/references.proto\"\x80\x01\n\x1aSearchReferenceSetsRequest\x12\x13\n\x0bmd5checksum\x18\x01 \x01(\t\x12\x11\n\taccession\x18\x02 \x01(\t\x12\x13\n\x0b\x61ssembly_id\x18\x03 \x01(\t\x12\x11\n\tpage_size\x18\x04 \x01(\x05\x12\x12\n\npage_token\x18\x05 \x01(\t\"c\n\x1bSearchReferenceSetsResponse\x12+\n\x0ereference_sets\x18\x01 \x03(\x0b\x32\x13.ga4gh.ReferenceSet\x12\x17\n\x0fnext_page_token\x18\x02 \x01(\t\"2\n\x16GetReferenceSetRequest\x12\x18\n\x10reference_set_id\x18\x01 \x01(\t\"\x82\x01\n\x17SearchReferencesRequest\x12\x18\n\x10reference_set_id\x18\x01 \x01(\t\x12\x13\n\x0bmd5checksum\x18\x02 \x01(\t\x12\x11\n\taccession\x18\x03 \x01(\t\x12\x11\n\tpage_size\x18\x04 \x01(\x05\x12\x12\n\npage_token\x18\x05 \x01(\t\"Y\n\x18SearchReferencesResponse\x12$\n\nreferences\x18\x01 \x03(\x0b\x32\x10.ga4gh.Reference\x12\x17\n\x0fnext_page_token\x18\x02 \x01(\t\"+\n\x13GetReferenceRequest\x12\x14\n\x0creference_id\x18\x01 \x01(\t\"a\n\x19ListReferenceBasesRequest\x12\x14\n\x0creference_id\x18\x01 \x01(\t\x12\r\n\x05start\x18\x02 \x01(\x03\x12\x0b\n\x03\x65nd\x18\x03 \x01(\x03\x12\x12\n\npage_token\x18\x04 \x01(\t\"W\n\x1aListReferenceBasesResponse\x12\x0e\n\x06offset\x18\x01 \x01(\x03\x12\x10\n\x08sequence\x18\x02 \x01(\t\x12\x17\n\x0fnext_page_token\x18\x03 \x01(\t2\xa5\x03\n\x10ReferenceService\x12\\\n\x13SearchReferenceSets\x12!.ga4gh.SearchReferenceSetsRequest\x1a\".ga4gh.SearchReferenceSetsResponse\x12\x45\n\x0fGetReferenceSet\x12\x1d.ga4gh.GetReferenceSetRequest\x1a\x13.ga4gh.ReferenceSet\x12S\n\x10SearchReferences\x12\x1e.ga4gh.SearchReferencesRequest\x1a\x1f.ga4gh.SearchReferencesResponse\x12<\n\x0cGetReference\x12\x1a.ga4gh.GetReferenceRequest\x1a\x10.ga4gh.Reference\x12Y\n\x12ListReferenceBases\x12 .ga4gh.ListReferenceBasesRequest\x1a!.ga4gh.ListReferenceBasesResponseb\x06proto3')
  ,
  dependencies=[ga4gh_dot_references__pb2.DESCRIPTOR,])
_sym_db.RegisterFileDescriptor(DESCRIPTOR)




_SEARCHREFERENCESETSREQUEST = _descriptor.Descriptor(
  name='SearchReferenceSetsRequest',
  full_name='ga4gh.SearchReferenceSetsRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='md5checksum', full_name='ga4gh.SearchReferenceSetsRequest.md5checksum', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='accession', full_name='ga4gh.SearchReferenceSetsRequest.accession', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='assembly_id', full_name='ga4gh.SearchReferenceSetsRequest.assembly_id', index=2,
      number=3, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='page_size', full_name='ga4gh.SearchReferenceSetsRequest.page_size', index=3,
      number=4, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='page_token', full_name='ga4gh.SearchReferenceSetsRequest.page_token', index=4,
      number=5, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=65,
  serialized_end=193,
)


_SEARCHREFERENCESETSRESPONSE = _descriptor.Descriptor(
  name='SearchReferenceSetsResponse',
  full_name='ga4gh.SearchReferenceSetsResponse',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='reference_sets', full_name='ga4gh.SearchReferenceSetsResponse.reference_sets', index=0,
      number=1, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='next_page_token', full_name='ga4gh.SearchReferenceSetsResponse.next_page_token', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=195,
  serialized_end=294,
)


_GETREFERENCESETREQUEST = _descriptor.Descriptor(
  name='GetReferenceSetRequest',
  full_name='ga4gh.GetReferenceSetRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='reference_set_id', full_name='ga4gh.GetReferenceSetRequest.reference_set_id', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=296,
  serialized_end=346,
)


_SEARCHREFERENCESREQUEST = _descriptor.Descriptor(
  name='SearchReferencesRequest',
  full_name='ga4gh.SearchReferencesRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='reference_set_id', full_name='ga4gh.SearchReferencesRequest.reference_set_id', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='md5checksum', full_name='ga4gh.SearchReferencesRequest.md5checksum', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='accession', full_name='ga4gh.SearchReferencesRequest.accession', index=2,
      number=3, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='page_size', full_name='ga4gh.SearchReferencesRequest.page_size', index=3,
      number=4, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='page_token', full_name='ga4gh.SearchReferencesRequest.page_token', index=4,
      number=5, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=349,
  serialized_end=479,
)


_SEARCHREFERENCESRESPONSE = _descriptor.Descriptor(
  name='SearchReferencesResponse',
  full_name='ga4gh.SearchReferencesResponse',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='references', full_name='ga4gh.SearchReferencesResponse.references', index=0,
      number=1, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='next_page_token', full_name='ga4gh.SearchReferencesResponse.next_page_token', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=481,
  serialized_end=570,
)


_GETREFERENCEREQUEST = _descriptor.Descriptor(
  name='GetReferenceRequest',
  full_name='ga4gh.GetReferenceRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='reference_id', full_name='ga4gh.GetReferenceRequest.reference_id', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=572,
  serialized_end=615,
)


_LISTREFERENCEBASESREQUEST = _descriptor.Descriptor(
  name='ListReferenceBasesRequest',
  full_name='ga4gh.ListReferenceBasesRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='reference_id', full_name='ga4gh.ListReferenceBasesRequest.reference_id', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='start', full_name='ga4gh.ListReferenceBasesRequest.start', index=1,
      number=2, type=3, cpp_type=2, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='end', full_name='ga4gh.ListReferenceBasesRequest.end', index=2,
      number=3, type=3, cpp_type=2, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='page_token', full_name='ga4gh.ListReferenceBasesRequest.page_token', index=3,
      number=4, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=617,
  serialized_end=714,
)


_LISTREFERENCEBASESRESPONSE = _descriptor.Descriptor(
  name='ListReferenceBasesResponse',
  full_name='ga4gh.ListReferenceBasesResponse',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='offset', full_name='ga4gh.ListReferenceBasesResponse.offset', index=0,
      number=1, type=3, cpp_type=2, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='sequence', full_name='ga4gh.ListReferenceBasesResponse.sequence', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='next_page_token', full_name='ga4gh.ListReferenceBasesResponse.next_page_token', index=2,
      number=3, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=716,
  serialized_end=803,
)

_SEARCHREFERENCESETSRESPONSE.fields_by_name['reference_sets'].message_type = ga4gh_dot_references__pb2._REFERENCESET
_SEARCHREFERENCESRESPONSE.fields_by_name['references'].message_type = ga4gh_dot_references__pb2._REFERENCE
DESCRIPTOR.message_types_by_name['SearchReferenceSetsRequest'] = _SEARCHREFERENCESETSREQUEST
DESCRIPTOR.message_types_by_name['SearchReferenceSetsResponse'] = _SEARCHREFERENCESETSRESPONSE
DESCRIPTOR.message_types_by_name['GetReferenceSetRequest'] = _GETREFERENCESETREQUEST
DESCRIPTOR.message_types_by_name['SearchReferencesRequest'] = _SEARCHREFERENCESREQUEST
DESCRIPTOR.message_types_by_name['SearchReferencesResponse'] = _SEARCHREFERENCESRESPONSE
DESCRIPTOR.message_types_by_name['GetReferenceRequest'] = _GETREFERENCEREQUEST
DESCRIPTOR.message_types_by_name['ListReferenceBasesRequest'] = _LISTREFERENCEBASESREQUEST
DESCRIPTOR.message_types_by_name['ListReferenceBasesResponse'] = _LISTREFERENCEBASESRESPONSE

SearchReferenceSetsRequest = _reflection.GeneratedProtocolMessageType('SearchReferenceSetsRequest', (_message.Message,), dict(
  DESCRIPTOR = _SEARCHREFERENCESETSREQUEST,
  __module__ = 'ga4gh.reference_service_pb2'
  # @@protoc_insertion_point(class_scope:ga4gh.SearchReferenceSetsRequest)
  ))
_sym_db.RegisterMessage(SearchReferenceSetsRequest)

SearchReferenceSetsResponse = _reflection.GeneratedProtocolMessageType('SearchReferenceSetsResponse', (_message.Message,), dict(
  DESCRIPTOR = _SEARCHREFERENCESETSRESPONSE,
  __module__ = 'ga4gh.reference_service_pb2'
  # @@protoc_insertion_point(class_scope:ga4gh.SearchReferenceSetsResponse)
  ))
_sym_db.RegisterMessage(SearchReferenceSetsResponse)

GetReferenceSetRequest = _reflection.GeneratedProtocolMessageType('GetReferenceSetRequest', (_message.Message,), dict(
  DESCRIPTOR = _GETREFERENCESETREQUEST,
  __module__ = 'ga4gh.reference_service_pb2'
  # @@protoc_insertion_point(class_scope:ga4gh.GetReferenceSetRequest)
  ))
_sym_db.RegisterMessage(GetReferenceSetRequest)

SearchReferencesRequest = _reflection.GeneratedProtocolMessageType('SearchReferencesRequest', (_message.Message,), dict(
  DESCRIPTOR = _SEARCHREFERENCESREQUEST,
  __module__ = 'ga4gh.reference_service_pb2'
  # @@protoc_insertion_point(class_scope:ga4gh.SearchReferencesRequest)
  ))
_sym_db.RegisterMessage(SearchReferencesRequest)

SearchReferencesResponse = _reflection.GeneratedProtocolMessageType('SearchReferencesResponse', (_message.Message,), dict(
  DESCRIPTOR = _SEARCHREFERENCESRESPONSE,
  __module__ = 'ga4gh.reference_service_pb2'
  # @@protoc_insertion_point(class_scope:ga4gh.SearchReferencesResponse)
  ))
_sym_db.RegisterMessage(SearchReferencesResponse)

GetReferenceRequest = _reflection.GeneratedProtocolMessageType('GetReferenceRequest', (_message.Message,), dict(
  DESCRIPTOR = _GETREFERENCEREQUEST,
  __module__ = 'ga4gh.reference_service_pb2'
  # @@protoc_insertion_point(class_scope:ga4gh.GetReferenceRequest)
  ))
_sym_db.RegisterMessage(GetReferenceRequest)

ListReferenceBasesRequest = _reflection.GeneratedProtocolMessageType('ListReferenceBasesRequest', (_message.Message,), dict(
  DESCRIPTOR = _LISTREFERENCEBASESREQUEST,
  __module__ = 'ga4gh.reference_service_pb2'
  # @@protoc_insertion_point(class_scope:ga4gh.ListReferenceBasesRequest)
  ))
_sym_db.RegisterMessage(ListReferenceBasesRequest)

ListReferenceBasesResponse = _reflection.GeneratedProtocolMessageType('ListReferenceBasesResponse', (_message.Message,), dict(
  DESCRIPTOR = _LISTREFERENCEBASESRESPONSE,
  __module__ = 'ga4gh.reference_service_pb2'
  # @@protoc_insertion_point(class_scope:ga4gh.ListReferenceBasesResponse)
  ))
_sym_db.RegisterMessage(ListReferenceBasesResponse)


# @@protoc_insertion_point(module_scope)



"""
Data-driven tests for ontologies.
"""
from __future__ import division
from __future__ import print_function
from __future__ import unicode_literals

import os.path

# TODO it may be a bit circular to use obo_parser as our method of
# accessing ontology information, since this is the method we use
# in the main code. However, other libraries have very heavy dependencies.
import ga4gh.datamodel.obo_parser as obo_parser

import ga4gh.protocol as protocol
import ga4gh.datamodel.ontologies as ontologies

import tests.datadriven as datadriven
import tests.paths as paths
from ga4gh.metadata_pb2 import OntologyTerm


def testReferenceSets():
    testDataDir = os.path.join(paths.testDataDir, "ontologies")
    pattern = "*.obo"
    for test in datadriven.makeTests(
            testDataDir, OntologyTest, pattern):
        yield test


class OntologyTest(datadriven.DataDrivenTest):
    """
    Data driven test class for ontologies.
    """
    def __init__(self, localId, oboFile):
        super(OntologyTest, self).__init__(localId, oboFile)
        self._oboReader = obo_parser.OBOReader(obo_file=oboFile)

    def getDataModelInstance(self, localId, dataPath):
        ontology = ontologies.Ontology(localId)
        ontology.populateFromFile(dataPath)
        return ontology

    def testProtocolElementValid(self):
        # We don't have a protocol element here so this isn't meaningful.
        pass

    def testGoodMappings(self):
        ontology = self._gaObject
        for term in self._oboReader:
            self.assertIn(term.id, ontology.getTermIds(term.name))
            gaTerm = ontology.getGaTermByName(term.name)
            self.assertTrue(protocol.validate(protocol.toJson(gaTerm),
                                              OntologyTerm))
            self.assertEqual(gaTerm.term, term.name)
            self.assertIn(gaTerm.id, ontology.getTermIds(term.name))
            self.assertEqual(
                gaTerm.source_version, ontology.getSourceVersion())
            self.assertEqual(gaTerm.source_name, ontology.getName())

    def testBadMappings(self):
        for badName in ["Not a term", None, 1234]:
            self.assertEqual(0, len(self._gaObject.getTermIds(badName)))



"""
A script to generate the schemas for the GA4GH protocol. These are generated
from a copy of the Protocol Buffers schema and use it to generate
the Python class definitions. These are also stored in revision
control to aid Travis building.
"""

from __future__ import division
from __future__ import print_function
from __future__ import unicode_literals


import os
import os.path
import subprocess
import fnmatch
import re
import argparse


class ProtobufGenerator(object):
    def __init__(self, version):
        self.version = version

    def run(self, args):
        script_path = os.path.dirname(os.path.realpath(__file__))
        server = os.path.realpath(os.path.join(script_path, ".."))
        schemas = os.path.realpath(args.schemapath)

        if not os.path.exists(schemas):
            raise Exception(
                "Can't find schemas folder. " +
                "Thought it would be at %s" % os.path.realpath(schemas))

        src = os.path.realpath(os.path.join(schemas, "src/main/proto"))

        if not os.path.exists(src):
            raise Exception(
                "Can't find source proto directory %s" % os.path.realpath(src))

        def find_in_path(cmd):
            PATH = os.environ.get("PATH", os.defpath).split(os.pathsep)
            for x in PATH:
                possible = os.path.join(x, cmd)
                if os.path.exists(possible):
                    return possible
            return None

        # From http://stackoverflow.com/a/1714190/320546
        def version_compare(version1, version2):
            def normalize(v):
                return [int(x) for x in re.sub(r'(\.0+)*$', '', v).split(".")]
            return cmp(normalize(version1), normalize(version2))

        protocs = [
            os.path.realpath(x) for x in
            "%s/protobuf/src/protoc" % server,
            find_in_path("protoc")
            if x is not None]
        protoc = None
        for c in protocs:
            if not os.path.exists(c):
                continue
            output = subprocess.check_output([c, "--version"]).strip()
            try:
                (lib, version) = output.split(" ")
                if lib != "libprotoc":
                    raise Exception("lib didn't match 'libprotoc'")
                if version_compare("3.0.0", version) > 0:
                    raise Exception("version < 3.0.0")
                protoc = c
                break

            except Exception:
                print (
                    "Not using {path} because it returned " +
                    "'{version}' rather than \"libprotoc <version>\", where " +
                    "<version> >= 3.0.0").format(path=c, format=output)

        if protoc is None:
            raise Exception("Can't find a good protoc. Tried %s" % protocs)
        print ("Using %s for protoc" % protoc)

        protos = []
        for root, dirs, files in os.walk(src):
            protos.extend([
                os.path.join(root, f)
                for f in fnmatch.filter(files, "*.proto")])
        if len(protos) == 0:
            raise Exception("Didn't find any proto files in %s" % src)
        cmd = "{protoc} -I {src} --python_out={server} {proto_files}".format(
            protoc=protoc, src=src,
            server=server, proto_files=" ".join(protos))
        os.system(cmd)

        with open("ga4gh/_protocol_version.py", "w") as version_file:
            version_file.write(
                "# File generated by scripts/process_schemas.py; "
                "do not edit\n")
            version_file.write("version = '{}'\n".format(self.version))


def main():
    parser = argparse.ArgumentParser(
        description="Script to process GA4GH Protocol buffer schemas")
    parser.add_argument(
        "version", help="Version number of the schema we're compiling")
    parser.add_argument(
        "schemapath",
        help="Path to schemas.")
    args = parser.parse_args()
    pb = ProtobufGenerator(args.version)
    pb.run(args)

if __name__ == "__main__":
    main()
